/***************************************************************************
    copyright            : (C) 2004 - 2005 by Marcin Szpyrka
    email                : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ccovergraph.h"


//--------------------------------------------------------------------------
/** Returns the coverability graph description. */

QString CCoverGraph::graphDescription()
{
  QString info("");
  
  for(cs = states.first();  cs != NULL; cs= states.next())
  {
    if(withTime)
    {
      info += cs->info(true);
      info += cs->successors(cnet, true) + "\n";
    }
    else
    {
      info += cs->info(false);
      info += cs->successors(cnet, false) + "\n";
    }
  }
  if(!finished)
  {
    info += "...(there is more states)\n";
  }
  return info;
}


//--------------------------------------------------------------------------
/** Computes the reachability graph using times. */
/*
void CCoverGraph::analyseT()
{
  CCoverState* curState = new CCoverState(0, placeNumber, transNumber);
  int i, j;
  
  for(i = 0; i < placeNumber; i++)
  {
    curState->setMarking(i,initMarking[i]);
  }	
	
  // szukamy aktywnych przejsc  
  int* active = new int[transNumber];
  int* nactive = new int[transNumber];
  int* delta= new int[transNumber];
 
  for(i = 0; i < transNumber; i++)
  {
    active[i]=1;
    for(j = 0; j < placeNumber; j++)
    {
      if(curState->marking(j) < inM[j][i])
      {
        active[i] = 0;
        break;
      }
    }
      
    if(active[i] > 0)
    {
      curState->setDelta(i,time[i]);
    }
    else
    {
      curState->setDelta(i,-1);
    }
  }	
 
  // wstawienie stanu poczatkowego na liste
  states.append(curState);
  stateIndex = 0;  
  int index;
  int minTime = -1;

 // wyliczanie grafu osiagalnosci	
 
  QPtrListIterator<CState> it(states);
  
  while((cs=it.current()) != NULL  && stateIndex <maxStNumber )
  {		
    curState = (CCoverState*)cs;
    bool found= false;
    minTime = -1;
    // skopiowanie aktywnych przejsc    
    for(i = 0; i < transNumber; i++)
    {
      active[i] = curState->delta(i);
      //szukamy najkrotszego mozliwego czasu
      if(active[i] >= 0) 
      { 
        found = true;
        if(minTime < 0)
        {
	        minTime=active[i];
	      }
	      else
        {
	        if(minTime >= active[i])
          {
	          minTime = active[i];
	        }
	      }
	    }
    }
	
    //tworzymy nowe stany na podstawie wybranych przejsc 
    index = 0;
    while(found && (index < transNumber)) 
    {
	    if((active[index]==minTime )) //przejscie do wykonania
	    {
	      CCoverState* newState = new CCoverState(-1, placeNumber, transNumber); // nowy stan
        newState->setParent(curState);
        int k;
        // wyliczamy nowe znakowanie
        for(i = 0; i < placeNumber; i++)
	      {
		      if(curState->marking(i) >= 0)
          {
            k = curState->marking(i);
            k = k - inM[i][index] + outM[i][index];
		      }
		      else
          {
		        k = curState->marking(i); //=-1 
		      }
		      newState->setMarking(i,k);
	     }			
	     
	     // wyliczamy nowy wektor opoznien dynamicznych
	     for(i = 0; i < transNumber; i++)
	     {
         nactive[i] = 1;
         for(j = 0; j < placeNumber; j++)
		     {
		       if((newState->marking(j) < inM[j][i] ) && 
			       (newState->marking(j) >=0 ))
           {
             nactive[i]= 0;
             break;
		       }
		     }
		   }
	     
	     for(i = 0; i < transNumber; i++)
	     {
		     if(nactive[i] > 0)
		     {
		       if(active[i] >= 0)
		       {
			       if(i != index)
             {
               newState->setDelta(i,active[i]-minTime);
             }
			       else
			       {
			         newState->setDelta(i,time[i]);
			       }
		       }
		       else
		       {
			       newState->setDelta(i,time[i]);
		       }
		     }
		   else
		   {
		     newState->setDelta(i,-1);
		   }
	   }

     //sprawdzamy czy dany stan pokrywa wczesniejsze
     newState->checkCover();
     // sprawdzamy, czy taki stan jest na liscie
     QPtrListIterator<CState> it2(states);
     CState* cs2;
     bool inList = false;
     int findState=-1;
     for(; (cs2= it2.current() )!= NULL; ++it2)
	   {
       if(cs2->equal(newState)) 
       {
         inList = true;
         findState = cs2->tag();
       }
	   }

	   // dodajemy nowy stan do listy
	   if(inList == false)	
	   {
		   newState->setTag(++stateIndex);
		   curState->addSucc(index, stateIndex); // dodajemy nastepnik
		   states.append(newState);
	   }
	   else
	   {
		   curState->addSucc(index, findState); // dodajemy nastepnik
		   delete newState;
	   }
	 }//if przejscie bylo wybrane
	 
	 index++;
	 
       }//while(found && index < transNumber)
     
     ++it;
  }
 
  if(cs == NULL)
  {
    finished=true;
  }
 
  //sprzatamy po sobie 
  delete active;
  delete nactive;
  delete delta;
}*/


//--------------------------------------------------------------------------
/** Computes the coverability graph (time is not used). */

void CCoverGraph::createGraphWithoutTime()
{
  CCoverState* curState = new CCoverState(0, placeNumber, transNumber);
  int i, j, k;
  
  for(i = 0; i < placeNumber; i++)
  {
    curState->setMarking(i, initMarking[i]);
  }
  
  // find active transitions
  int* active  = new int[transNumber];
  int* nactive = new int[transNumber];
  int* delta   = new int[transNumber];
  
  for(i = 0; i < transNumber; i++)
  {
    active[i] = 1;
    for(j = 0; j < placeNumber; j++)
	  {
	    if(curState->marking(j) < outM[j][i])
      {
        active[i] = 0;
        break;
      }
    }

    if(active[i] > 0)
    {
      curState->setDelta(i,0);
    }
    else
    {
      curState->setDelta(i,-1);
    }
  }	
	 
  // append the initial state to the list 
  states.append(curState);
  stateIndex = 0;   // number of the current state
  int index;

  // generate the coverability graph
  
  QPtrListIterator<CState> it(states);

  while(((cs = it.current()) != NULL)  && stateIndex < maxStNumber)
  {		
    curState = (CCoverState*)cs;
    bool found = false;
    
    // copy enable transitions    
    for(i = 0; i < transNumber; i++)
    {
      active[i] = curState->delta(i);
      // check whether there exists at least one enable transition
      if(active[i] >= 0)
      { 
        found=true;
      }
    }
    
    if(withPriorities == true) // check priorities  
    {
      for(i = 0; i < transNumber - 1; i++)
      {
        for(j = i + 1; j < transNumber; j++) 
        {
          if((active[i] >= 0) && (active[j] >= 0) 
            && (priority[i] != priority[j]))
          {
            for(k = 0; k < placeNumber; k++)
            {
              if((outM[k][i] > 0) && (outM[k][j] > 0) 
                && (outM[k][i] + outM[k][j] > curState->marking(k)))
              {
                if(priority[i] > priority[j])
                {
                  active[j] = -1;
                }
                else
                {
                  active[i] = -1;
                }
              }
            }
          }
        }
      } 
    }
		
    // generate the new states 
    index = 0;
    while(found && (index < transNumber)) 
	  {
	    if(active[index] >= 0) 
	    {	      
	      CCoverState* newState = new CCoverState(-1, placeNumber, transNumber); // new state
	      newState->setParent(curState);
	      int k;
        
	      // new marking
	      for(i = 0; i < placeNumber; i++)
	      {
		      if(curState->marking(i) >= 0)
          {
            k = curState->marking(i);
            k = k - outM[i][index] + inM[i][index];
          }
          else
          {
		        k = curState->marking(i); //=-1 
		      }
		      newState->setMarking(i,k);
	      }
        
        // modify the markings if it covers another one
        newState->checkCover(); 
					    
	      // new dynamic delays vector
	      for(i = 0; i < transNumber; i++)
	      {
          nactive[i] = 1;
          for( j = 0; j < placeNumber; j++)
		      {
		        if((newState->marking(j) < outM[j][i] ) && 
			        (newState->marking(j) >= 0))
            {
		          nactive[i]= 0;
		          break;
		        }
		      }

		      if(nactive[i] == 1)
          {
		        newState->setDelta(i,0);
		      }
		      else
          {
		        newState->setDelta(i,-1);
		      }
		    }
        
        // check whether such a state already exists
        QPtrListIterator<CState> it2(states);
        CState* cs2;
        bool inList = false;
        int findState = -1;
        for(; (cs2 = it2.current()) != NULL; ++it2)
        {
          if(cs2->equal(newState)) 
          {
            inList = true;
            findState = cs2->tag();
          }
        }  
				
	      // add the new state to the list
	      if(inList == false)	
        {
          newState->setTag(++stateIndex);
          curState->addSucc(index, stateIndex); // add successor
          states.append(newState);
        }
        else
        {
          curState->addSucc(index, findState); // add successor
          delete newState;
        }
	      
	    } // end if(active[index] >= 0) 		    
	    index++;
	 
	  } // end while(found && index < transNumber)      
    ++it;       
  } // end while
	
  if(cs == NULL)
  {
    finished = true;
  }

  delete active;
  delete nactive;
  delete delta;   
}


//--------------------------------------------------------------------------
/** Creates coverability graph. */

void CCoverGraph::createGraph()
{
  if(empty) return;
  
  finished = false;  
  
/*  if(withTime)
  {
    createGraphWithTime();
  } 
  else 
  { */
    createGraphWithoutTime();
//  }
}






