/***************************************************************************
    copyright            : (C) 2004 - 2005 by Marcin Szpyrka
    email                : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ccoverstate.h"


//--------------------------------------------------------------------------
/** Sets the marking of the place with number \a index to \a tokens,
  or does nothing if the index is out of range or the new marking is negative.
	The valid range is 0..(placeCount() - 1) inclusive. */ 

void CCoverState::setMarking(int index, int tokens)
{
	if((index < pCount) && (tokens >= -1))
	{
		mark[index] = tokens;
	}
}


//--------------------------------------------------------------------------
/** Function checks if there exists a node on a path from the root
    state that is covered by the current node. 
    If it finds such a node it modifies the marking (oo). */

void CCoverState::checkCover()
{  
  CCoverState* p = parent;

  while(p != NULL)
  {
    if(isCovered(p))  //we find covered state
    { 
      for(int i = 0; i < pCount; i++)
      {
	      if((mark[i] > p->mark[i]))
        {
	        mark[i]=-1;
	      }
      }
      break; 
    }
    p = p->getParent();
  }
}


//--------------------------------------------------------------------------
/** Returns \a true if the state covers the state \a cst. */
 
bool CCoverState::isCovered(CCoverState* cst)
{
  if(cst != NULL)
  {
    if(pCount != cst->pCount) return false;
    
    //check marking
    for(int i = 0; i < pCount; i++)
    {
      if((mark[i] < cst->mark[i]) && (mark[i] >= 0)) 
      {
	      return false;
      }  
      if((cst->mark[i] < 0) && (mark[i] >= 0))
      {
	      return false;
      }  
    } 
    return true;
  }
  return false;
}


//--------------------------------------------------------------------------
/** Returns description of the state: state tag, marking and delay vector. */

QString CCoverState::info()
{
  QString desc("state: ");
  desc += QString::number(sTag);
  desc += "\nmarking: (";

  for(int i = 0; i < pCount; i++)
  {
    if(i != 0) desc += ", ";
    if(mark[i]>=0)
    {
      desc += QString::number(mark[i]);
    }
    else
    {
      desc += "oo"; // oo
    }  
  }
    
  desc += ")\n";

  return desc;
}
