/***************************************************************************
    copyright            : (C) 2004 - 2005 by Marcin Szpyrka
    email                : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CCOVERSTATE_H
#define CCOVERSTATE_H

#include "cstate.h"
#include "../model/cplace.h"
#include "../model/ctransition.h"

class CCoverState : public CState {
protected:
  CCoverState* parent;

  bool isCovered(CCoverState* cst);
  
public:
  /** Creates an empty state.	*/
  CCoverState() : CState(){ parent = NULL; }
  
  /** Creates a state with \a tag for a net with \a pc places and
  \a tc transitions.	*/
  CCoverState(int tag, int pc, int tc) : CState(tag,  pc, tc){ parent = NULL; }
  
  /** Destroys the state.	*/
  ~CCoverState(){}
  
  /** Creates a new state that is copy of \a source but it contains
  na empty list of successors. The \a source state is set as a parent
  of the state. */
  CCoverState(CCoverState &source) : CState(source){ parent = & source; }  
  
  /** Sets the parent state to \a p.
    If \a p = \a null, the state has no parent. */ 
  void setParent(CCoverState* p){ parent = p; }
    
  /** Returns the parent state of the state.
    If the result is equal to \a null the state has no parent. */  
  CCoverState * getParent(){ return parent; }

  void checkCover();
  QString info();
  void setMarking(int index, int tokens);
};




#endif
