/***************************************************************************
    copyright            : (C) 2004 - 2005 by Marcin Szpyrka
    email                : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include"cfraction.h"

//--------------------------------------------------------------------------
/** Check the sign of the fraction and make the denominator sign positive. */

void CFraction::sign()
{
  if(denominator < 0)
  {
    numerator = -numerator;
    denominator = -denominator;
  }
}


//--------------------------------------------------------------------------
/** Abridges the fraction. */

void CFraction::abridge()
{
  int i, j;
  if((abs(numerator) > 1) && (abs(denominator) > 1))
  {
    i = abs(numerator) < abs(denominator) ? abs(numerator) : abs(denominator);
    for(j = i; j > 1; j--)
    {
      if((numerator % j == 0) && (denominator % j == 0))
      {
        numerator /= j;
        denominator /= j;
        break;  // wykonalismy dzielenie przez najwiekszy 
                // wspolny dzielnik
      }
    }
  }
}


//--------------------------------------------------------------------------
/** Addition operator. */

CFraction CFraction::operator+(const CFraction &x)
{
  CFraction result;
  result.denominator = denominator * x.denominator;
  result.numerator = numerator * x.denominator + x.numerator * denominator;
  result.abridge();
  result.sign();
  return result;
}


//--------------------------------------------------------------------------
/** Subtraction operator. */

CFraction CFraction::operator-(const CFraction &x)
{
  CFraction result;
  result.denominator = denominator * x.denominator;
  result.numerator = numerator * x.denominator - x.numerator * denominator;
  result.abridge();
  result.sign();
  return result;
}


//--------------------------------------------------------------------------
/** Multiplication operator. */

CFraction CFraction::operator*(const CFraction &x)
{
  CFraction result;
  result.denominator = denominator * x.denominator;
  result.numerator = numerator * x.numerator;
  result.abridge();
  result.sign();
  return result;
}


//--------------------------------------------------------------------------
/** Division operator. */

CFraction CFraction::operator/(const CFraction &x)
{
  CFraction result;
  result.denominator = denominator * x.numerator;
  result.numerator = numerator * x.denominator;
  result.abridge();
  result.sign();
  return result;
}


//--------------------------------------------------------------------------
/** Returns text description of the fraction. */

QString CFraction::text()
{
  QString temp;
  temp = QString::number(numerator);
  
  if(numerator == 0) return temp;
  
  if(denominator != 1) 
  {
    temp = temp + "/" + QString::number(denominator);
  }
  
  return temp;
}



