/***************************************************************************
    copyright            : (C) 2004 - 2005 by Marcin Szpyrka
    email                : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
 
#include "cmatrix.h"
#include <qstrlist.h> 
 
//-------------------------------------------------------------------------
/** Constructs an empty matrix with \a numRows rows and
    \a numCols columns. */
 
CMatrix::CMatrix(int numRows, int numCols)
{
  rows = numRows;
  cols = numCols;
  int i, j;
  
  data = new CFraction*[rows];
  for(i = 0; i < rows; i++)
  {
    data[i] = new CFraction[cols];
  }
  
  for(i = 0; i < rows; i++)
  {
    for(j = 0; j < cols; j++)
    {
      data[i][j] = CFraction(0);
    }
  }
}
 
 
//-------------------------------------------------------------------------
/** Constructs a matrix with the same values as the \a source matrix. */ 
 
CMatrix::CMatrix(CMatrix &source)
{
  rows = source.rowsCount();
  cols = source.colsCount();
  int i, j;
  
  data = new CFraction*[rows];
  for(i = 0; i < rows; i++)
  {
    data[i] = new CFraction[cols];
  }
  
  for(i = 0; i < rows; i++)
  {
    for(j = 0; j < cols; j++)
    {
      data[i][j] = source.data[i][j];
    }
  } 
}
 
 
//-------------------------------------------------------------------------
/** Destroys the matrix. */ 
 
CMatrix::~CMatrix()
{
  int i;
  for(i = 0; i < rows; i++)
  {
    delete [] data[i];
  }
  delete [] data;
  
  if(emptyRow) delete [] emptyRow;
  if(emptyCol) delete [] emptyCol;
  if(colIndex) delete [] colIndex;
}
 

//-------------------------------------------------------------------------
/** Assignment operator. */
  
CMatrix & CMatrix::operator=(CMatrix &source)
{
  int i, j;
  
  if(this != &source)  
  { 
    for(i = 0; i < rows; i++)
    {
      delete [] data[i];
    }
    delete [] data;
    
    rows = source.rowsCount();
    cols = source.colsCount();
    
    data = new CFraction*[rows];
    for(i = 0; i < rows; i++)
    {
      data[i] = new CFraction[cols];
    }
    
    for(i = 0; i < rows; i++)
    {
      for(j = 0; j < cols; j++)
      {
        data[i][j] = source.data[i][j];
      }
    }
  }
  return *this;
}


//-------------------------------------------------------------------------
/** Multiplication operator. Returns 0x0 matrix if the argumnet matrices
    cannot be multiplied. */

CMatrix & CMatrix::operator*(const CMatrix &m)
{
  if(cols != m.rowsCount()) 
  {
    CMatrix* res = new CMatrix(0, 0);
    return *res;
  }
  
  int i, j, k;
  int mRows, mCols;
  CFraction frac;
  mRows = m.rowsCount();
  mCols = m.colsCount();
  
  CMatrix* res = new CMatrix(rows, mCols);
  
  for(i = 0; i < rows; i++)
  {
    for(j = 0; j < mCols; j++)
    {
      frac = CFraction(0,1);
      for(k = 0; k < cols; k++)
      {
        frac = frac + data[i][k] * m.data[k][j];
      }
      res->data[i][j] = frac;
    }
  }
  
  return *res;
}



//-------------------------------------------------------------------------
/** Extends the matrix with the identity matrix. */

void CMatrix::extend()
{
  CMatrix *m;
  int i, j;
  
  m = new CMatrix(rows, cols + rows);
  for(i = 0; i < rows; i++)
  {
    for(j = 0; j < cols; j++)
    {
      m->data[i][j] = data[i][j];
    }
  }
  
  int c = cols + rows;
  for(i = 0; i < rows; i++)
  {
    for(j = cols; j < c; j++)
    {
      m->data[i][j] = CFraction(0);
    }
  }
  for(i = 0; i < rows; i++)
  {
    m->data[i][cols + i] = CFraction(1);
  }
}


 //-------------------------------------------------------------------------
/** Returns text description of the matrix. */

QString CMatrix::text()
{
  QString temp;
  QString result = "";
  int maxLength = 0;
  int i, j, k;
  
  for(i = 0; i < rows; i++)
  {
    for(j = 0; j < cols; j++)
    {
      temp = data[i][j].text();
      k = temp.length();
      if(k > maxLength) maxLength = k;
    }
  }
  
  maxLength++;
  for(i = 0; i < rows; i++)
  {
    for(j = 0; j < cols; j++)
    {
      temp = data[i][j].text();
      k = temp.length();
      while(k < maxLength) 
      {
        temp = " " + temp;
        k++;
      }
      result += temp;
    }
    result += "\n";
  }
  
  return result;
}


//-------------------------------------------------------------------------
/** Multiplayes the \a row by \a frac. */ 
 
void CMatrix::multiply(int row, CFraction frac) 
{ 
  for(int i = 0; i < cols; i++)
  {
    data[row][i] = data[row][i] * frac;
  }    
} 
 


//-------------------------------------------------------------------------
/** Divides the \a row by \a frac. */ 
 
void CMatrix::divide(int row, CFraction frac) 
{ 
  for(int i = 0; i < cols; i++)
  {
    data[row][i] = data[row][i] / frac;
  }    
}

 
//------------------------------------------------------------------------- 
/** Swaps two rows. */ 
 
void CMatrix::swapRows(int row1, int row2) 
{
  CFraction frac;
  int i;
  
  for(i = 0; i < cols; i++)
  {
    frac = data[row1][i];
    data[row1][i] = data[row2][i];
    data[row2][i] = frac;
  }
}
  
  
//------------------------------------------------------------------------- 
/** Swaps two columns. */ 

void CMatrix::swapColumns(int col1, int col2)
{
  CFraction frac;
  int i;
  
  for(i = 0; i < rows; i++)
  {
    frac = data[i][col1];
    data[i][col1] = data[i][col2];
    data[i][col2] = frac;
  }
}

 
//------------------------------------------------------------------------- 
/** Transforms the matrix to a base form. */ 
 
void CMatrix::solve()
{
  int i, j, k;
  
  // initialise arrays
  emptyRow = new int[rows];
  emptyCol = new int[cols];
  colIndex = new int[cols];
  for(i = 0; i < cols; i++)
  {
    colIndex[i] = i;
  }
  
  checkRows(); 
  checkColumns();
  
  i = 0;
  bool find;
  while((emptyCol[i] == 1) && (emptyRow[i] == 1))
  {
    // the (i,i) element must be != 0
    find = false; 
    while(find == false)
    {
      if(data[i][i].numerator == 0)
      {
        for(j = i + 1; j < rows; j++)
        {
          if(data[j][i].numerator != 0)
          {
            swapRows(i, j);
            find = true;
            break;
          }  
        }
      }
      else
      {
        find = true;
      }
      if(find == false) // we have to swap some columns
      {
        for(j = i + 1; j < cols; j++)
        {
          if(data[i][j].numerator != 0)
          {
            swapColumns(i, j);
            k = colIndex[i];
            colIndex[i] = colIndex[j];
            colIndex[j] = k;
            break;
          }
        }
      }
    }
    
    // the (i,i) element must be equal to 1
    if((data[i][i].numerator != 1) || (data[i][i].denominator != 1))
    {
      divide(i, data[i][i]);
    }
    
    // only the (i,i) element must be != 0
    for(j = 0; j < rows; j++)
    {
      if((data[j][i].numerator != 0) && (j != i))
      {
        subtract(j, i, data[j][i]);
      }
    }
     
    checkRows();
    checkColumns();
    i++;
  }
}
 
 
//------------------------------------------------------------------------- 
/** Finds and checks off empty rows and then moves them 
    to the bottom of the matrix. */ 
 
void CMatrix::checkRows() 
{
  int i, j, k;
  bool empty;
  
  for(i = 0; i < rows; i++)
  {
    empty = true; 
    for(j = 0; j < cols; j++)
    {
      if(data[i][j].numerator != 0) empty = false;
    }  
    if(empty) emptyRow[i] = 0;
    else emptyRow[i] = 1;
  }
  
  for(i = 0; i < rows; i++)
  {
    if(emptyRow[i] == 0)
    {
      for(j = i + 1; j < rows; j++)
      {
        if(emptyRow[j] == 1) 
        {
          swapRows(i, j);
          k = emptyRow[i];
          emptyRow[i] = emptyRow[j];
          emptyRow[j] = k;
          break;
        }
      }
    }
  }  
} 
 
 
//------------------------------------------------------------------------- 
/** Finds and checks off empty columns and then moves them 
    to the right of the matrix. */ 
 
void CMatrix::checkColumns() 
{
  int i, j, k;
  bool empty;
  
  for(i = 0; i < cols; i++)
  {
    empty = true; 
    for(j = 0; j < rows; j++)
    {
      if(data[j][i].numerator != 0) empty = false;
    }  
    if(empty) emptyCol[i] = 0;
    else emptyCol[i] = 1;
  }
  
  for(i = 0; i < cols; i++)
  {
    if(emptyCol[i] == 0)
    {
      for(j = i + 1; j < cols; j++)
      {
        if(emptyCol[j] == 1) 
        {
          swapColumns(i, j);
          k = colIndex[i];
          colIndex[i] = colIndex[j];
          colIndex[j] = k;
          break;
        }
      }
    }
  }  
} 
 
 
//------------------------------------------------------------------------- 
/** Subtracts two rows ( \a row1 - \a row2 ). */  
 
void CMatrix::subtract(int row1, int row2, CFraction frac) 
{
  int i;
  for(i = 0; i < cols; i++)
  {
    data[row1][i] = data[row1][i] - frac * data[row2][i];
  }
}


//------------------------------------------------------------------------- 
/** Returns text description of the computed invariants. */  

QString CMatrix::invariants()
{
  QString temp;
  int ord = order();
  
  if(ord == cols)
  {
    temp = "No invariants found.\n";
  }
  else
  {
    int i, j;
    QStrList results;
    
    // matrix M, change the sign
    for(i = ord; i < cols; i++)
    {
      j = 0;
      while(emptyRow[j] == 1)
      {
        data[j][i] = data[j][i] * CFraction(-1);
        j++;
      }
    }

    for(i = 0; i < ord; i++)
    {
      temp = "";
      for(j = ord; j < cols; j++)
      {
        if(data[i][j].numerator != 0)
        {
          if(temp != "") temp += " + ";
          if((data[i][j].numerator == 1) && (data[i][j].denominator == 1))
          {
            temp += "x";
            temp += QString::number(j - ord + 1);
          }
          else if ((data[i][j].numerator == -1) && (data[i][j].denominator == 1))
          {
            temp += "- x";
            temp += QString::number(j - ord + 1);
          }
          else
          {
            temp += data[i][j].text();
            temp += "*x";
            temp += QString::number(j - ord + 1);
          }
        }
      }
      if(temp == "") temp = "0";
      results.append(temp);
    }
    for(i = 0; i < cols - ord; i++)
    {
      temp = "x";
      temp += QString::number(i + 1);
      results.append(temp);
    }
    
    // Put the results in the proper order
    temp = "(";
    for(i = 0; i < cols; i++)
    {
      temp += results.at(colIndex[i]);
      if(i != (cols - 1)) temp += ", ";
    }
    temp += ")\n";
  }
  
  temp.replace("+ -", "-");
  return temp;
} 
 
 
//------------------------------------------------------------------------- 
/** Returns the matrix order. */  
 
int CMatrix::order() 
{ 
  int r, c;
  r = 0;
  c = 0;
  
  while((emptyRow[r] == 1) && (r < rows)) r++;
  while((emptyCol[c] == 1) && (c < cols)) c++;
  
  return r < c ? r : c;
} 
 
 
 
 
 
