/***************************************************************************
    copyright            : (C) 2004 - 2005 by Marcin Szpyrka
    email                : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef REAGRAPH_H
#define REAGRAPH_H

#include "cstate.h"
#include "../model/cnet.h"
#include <qstring.h>
#include <qptrlist.h>


class CReachGraph{
protected:
  CNet* cnet;
  QPtrList<CState> states;
  CState* cs;
  
  int stateIndex ;
  int maxStNumber;
  int placeNumber;
  int transNumber;
  
  int** inM;
  int** outM;
  int*  initMarking;
  int*  priority;
  int*  time;
  
  bool withTime;
  bool withPriorities;
  bool finished;
  bool empty;

  void clearList();
  void getInMatrix();
  void getOutMatrix();
  void getPriorities();
  void getTimes();
  void getInitialMarking();
  virtual void createGraphWithoutTime();
  virtual void createGraphWithTime();

public:
  CReachGraph(CNet* cnet);
  virtual ~CReachGraph();

  void setMaxStateNumber(int nr);  
  
  /** Returns the maximum number of states in the reachability 
    graph that will be computed. */
  int maxStateNumber(){ return maxStNumber; }

  /** Sets the hasPriority property - the priorities will be used while
    the reachability/coverability graph is generated. */
  void usePriorities(bool yes){ withPriorities = yes;}
  
  /** Sets the hasTimes property - the tims will be used while
    the reachability graph is generated. */
  void useTime(bool yes){ withTime = yes;};  

  virtual void createGraph();
  virtual QString graphDescription();
  
  /** Returns true if all graph states have been found. */
  bool isFinished(){ return finished; }
  
};

#endif
