/***************************************************************************
    copyright            : (C) 2004 - 2005 by Marcin Szpyrka
    email                : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CSTATE_H
#define CSTATE_H

#include "../model/cnet.h"
#include<qmap.h>
#include <qstring.h>


/**
	The CState represents a state of a simple time Petri net.
	It contains net marking and dynamic delay vector.
*/

class CState
{
protected:
     
	int sTag;
	int pCount;
	int tCount;
	int* mark;
	int* tdelta;
	QMap<int,int> succ;
    
public:
	CState();
	CState(int tag, int pc, int tc);
	virtual ~CState();
	CState(CState &source);
	bool equal(CState *source);
  
	/** Sets the state tag to \a tag. */
	void setTag(int tag){ sTag = tag; }
	
	/** Returns the tag of the state. */
	int tag(){ return sTag; }
	
	int marking(int index);
	int delta(int index);
	virtual void setMarking(int index, int tokens);
	void setDelta(int index, int ndelta);
	
	/** Add a new successor to the list of the state successors. */
	void addSucc(int t, int tag){ succ[t] = tag; }

	virtual QString info(bool withTime);
	QString successors(CNet* cnet, bool withTime);
};


#endif

