/***************************************************************************
    copyright            : (C) 2004 - 2005 by Marcin Szpyrka
    email                : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CPROPERTIESEDITOR_H
#define CPROPERTIESEDITOR_H

#include <qdockwindow.h>
#include <qwidgetstack.h>
#include <qtable.h>
#include <qlayout.h>
#include <qstring.h>
#include "../view/cnetview.h"

/** The CPropertiesEditor class provides a floating property editor.
  It contains a stack of tables of which only the top table
  is user-visible. */

class CPropertiesEditor : public QDockWindow
{
	Q_OBJECT
private:
  QWidgetStack*    widgetStack;
  QPtrList<QTable> tables;
  QTable*          curTable;
  int              enabled;

public:
	CPropertiesEditor(QWidget *parent, QString caption);

	/** Destroys the properties editor. */
	~CPropertiesEditor(){}

	int addTable(QString header1, QString header2, int rows);

	/** Returns a pointer to the table at position index in the
  properties editor, or 0 if the index is out of range.
  The valid range is 0..(tablecount() - 1) inclusive.*/
	QTable* table(int index){ return tables.at(index); }

	/** Returns the number of tables of the properties editor. */
	int count() { return tables.count(); }

public slots:
	void canvasSelected(CNetView*);
	void placeSelected(CEllipseWithLabels*);
	void transSelected(CRectangleWithLabels*);
	void arcSelected(CLineWithLabels*);
  void propertyChanged(int x, int y);

signals:
	void setCanvasProperties(QString n, int x, int y);
	void setPlaceProperties(QString name, int capacity, int marking, int tag);
	void setTransProperties(QString name, int priority, int time, int tag);
	void setArcProperties(int weight);
};



#endif





