
/***************************************************************************
    copyright            : (C) 2004 - 2005 by Marcin Szpyrka
    email                : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

//--------------------------------------------------------------------------

void mainForm::init()
{
	net = new CNet();
	options = new COptions();
	
	readSettings();
	createPropertyEditor();
	createWidgets();
	createConnections();
  
  /* Initialize variables. */
	modified = false;
	fileName = "";
  net->setSimulationParameters(!options->ignorePriorities, !options->ignoreTime);
  netView->setSimulationParameters(!options->ignorePriorities, !options->ignoreTime); 
	
	// set the group of buttons
	toolsActionGroup = new QActionGroup(this);
	toolsActionGroup->add(netArcAction);
	toolsActionGroup->add(netPlaceAction);
	toolsActionGroup->add(netTransitionAction);
	toolsActionGroup->add(netSelectAction);
	toolsActionGroup->add(simulationInteractiveRunAction);
	toolsActionGroup->add(simulationAutomaticRunAction);
	toolsActionGroup->add(simulationStopAction);
	toolsActionGroup->setExclusive(true);
	netSelectAction->setOn(true);
	
	/* Turn off disable (not implemented) actions. */
	editUndoAction->setEnabled(false);
	editRedoAction->setEnabled(false);
	editCutAction->setEnabled(false);
	editCopyAction->setEnabled(false);
	editPasteAction->setEnabled(false);
	alingmentHorizontalSpreadAction->setEnabled(false);
	alingmentVerticalSpreadAction->setEnabled(false);
  simulationAutomaticRunAction->setEnabled(false);
}


//--------------------------------------------------------------------------
/** Creates a new project. */

void mainForm::fileNew()
{
	int answer = 1;
	if(modified == true)
	{
		answer = QMessageBox::warning(this, "APE",
																	QString("File has changed, Do you want to save it?"),
																	"&Save", "&Abort", QString::null, 0, 1);
	}
	// Enter == button 0   Escape == button 1
	
	if(answer == 0)
	{
		fileSave();
	}
	
  netView->setSelectMode();
  netSelectAction->setOn(true);
	netView->removeAll();
	modified = false;
	fileName = "";
  setCaption(QString("APE"));  
}


//--------------------------------------------------------------------------

void mainForm::fileOpen()
{
	// check whether the current project is modified
	int answer = 1;
	if(modified == true)
		answer = QMessageBox::warning(this, "APE",
																	QString("File has changed, Do you want to save it?"),
																	"&Save", "&Abort", QString::null, 0, 1);
	// Enter == button 0   Escape == button 1
	if(answer == 0)
	{
		fileSave();
	}
	
	// choose a file to open
	QString newFileName = QFileDialog::getOpenFileName("", "APE model (*.xml)",
																										 this, "open file dialog", "Choose a file" );
	
	if(!newFileName.isEmpty())
	{
		QFile file(newFileName);
		if(file.open(IO_ReadOnly)) // True if the file is opened
		{
			// delete the old contents		
			netView->setSelectMode();
      netSelectAction->setOn(true);
      netView->removeAll();
			modified = false;
      fileName = newFileName;
      setCaption(QString("APE -- %1" ).arg(fileName));
			
			QXmlSimpleReader reader;
			CXMLReader handler(net, netView);
			reader.setContentHandler(&handler);
			reader.setErrorHandler(&handler);
			reader.parse(&file);
		}
	}
}


//--------------------------------------------------------------------------

void mainForm::fileSave()
{
	if(fileName.isEmpty())
	{
		fileSaveAs();
		return;
	}
	
	QFile file(fileName);
	if(file.open(IO_WriteOnly)) // True if the file is opened
	{
		CXMLGenerator out(&file);
		out.writeNet(netView);
    modified = false;
    setCaption(QString("APE -- %1" ).arg(fileName));
	}
}


//--------------------------------------------------------------------------

void mainForm::fileSaveAs()
{
	// Choose a file name.
	QString newFileName = QFileDialog::getSaveFileName("",
												"APE model (*.xml)", this, 
												"save file dialog", "Choose a file" );
	
	// A file name has been chosen.
	if(! newFileName.isEmpty())
	{
		int answer = 0;
    
    if(newFileName.right(4) != ".xml")
      newFileName += ".xml";
    
		if(QFile::exists(newFileName))
		{
			answer = QMessageBox::warning(this, "APE",
							 QString("File already exists, Do you want to overwrite it?"),
																		"&Yes", "&No", QString::null, 0, 1);
		}  // Enter == button 0   Escape == button 1
		
		if(answer == 0)
		{
			fileName = newFileName;
			fileSave();
		}
	}
}


//--------------------------------------------------------------------------

void mainForm::fileSaveReport()
{
	// Choose a file name.
	QString newFileName =
			QFileDialog::getSaveFileName("", "APE Report (*.txt)", this,
																	 "save file dialog", "Choose a file" );
	// A file name has been chosen.
  if(! newFileName.isEmpty())
	{
		int answer = 0;
    
    if(newFileName.right(4) != ".txt")
      newFileName += ".txt";
      
		if(QFile::exists(newFileName))
			answer = QMessageBox::warning(
					this, "APE",
					QString("File already exists, Do you want to overwrite it?"),
					"&Yes", "&No", QString::null, 0, 1);
		// Enter == button 0   Escape == button 1
		if(answer == 0)
		{
			QFile file(newFileName);
			if(file.open(IO_WriteOnly)) // True if the file is opened
			{
				QTextStream stream(&file);
				stream << reportTextEdit->text();
			}
		}
	}
}


//--------------------------------------------------------------------------

void mainForm::fileExport()
{
	QString newFileName;
	QString filter;
	QFileDialog *fileDialog = new QFileDialog("",
							"Scalable Vector Graphics (*.svg);;XFig File Format (*.fig)",
																						this, "Export File Dialog", TRUE);
	fileDialog->setMode(QFileDialog::AnyFile);
	
	// Choose a file name.
	if(fileDialog->exec() != QDialog::Accepted) return;
	
	newFileName = fileDialog->selectedFile();
	filter = fileDialog->selectedFilter();
	
	// A file name has been chosen.
	if(newFileName.isEmpty()) return;
	
	if(filter == "XFig File Format (*.fig)")
	{
		if(newFileName.find(".fig") == -1)
		{
			newFileName += ".fig";
		}
	}	
	else
	{	
		if(newFileName.find(".svg") == -1)
		{
			newFileName += ".svg";
		}
	}
	
	int answer = 0;
	if(QFile::exists(newFileName))
	{
		answer = QMessageBox::warning(this, "APE",
						 QString("File already exists, Do you want to overwrite it?"),
						 "&Yes", "&No", QString::null, 0, 1);
	}  // Enter == button 0   Escape == button 1
	
	if(answer != 0) // Not Yes
		return;
	
	QFile file(newFileName);
	if(file.open(IO_WriteOnly)) // True if the file is opened
	{
		if(filter == "XFig File Format (*.fig)")
		{
			CXfigExport exporter(file, options->xfigLineWidth, 
													 options->xfigOrientation + CXfigExport::Landscape,
													 options->xfigSize + CXfigExport::Letter,
													 options->xfigFillShapes);
			exporter.exportNet(canvas);
		}
		else // svg
		{
			SVGOutput::SVGExporter exporter(file);
			exporter.setExternalStyleSheet(options->svgStylesheet);
			exporter.setExternalScriptUrl(options->svgScript);
			
			QString temp = options->svgScriptType;
			if(temp == "(Empty)") temp = "";
			exporter.setExternalScriptType(temp);
			
			exporter.setWidth(options->svgCanvasWidth);
			exporter.setHeight(options->svgCanvasHeight);
			exporter.setViewBox(QRect(options->svgX, options->svgY, 
																options->svgViewWidth * 10, 
																options->svgViewHeight * 10));
			
			exporter.exportNet(canvas);
		}
	}
}


//--------------------------------------------------------------------------

void mainForm::fileExit()
{
	int answer = 1;
	if(modified == true)
	{
		answer = QMessageBox::warning(
				this, "APE", QString("File has changed, Do you want to save it?"),
				"&Save", "&Abort", QString::null, 0, 1);
	}
	
	if(answer == 0)
	{
		fileSave();
	}
	QApplication::exit(0);
}


//--------------------------------------------------------------------------

void mainForm::helpAbout()
{
	aboutForm *about = new aboutForm();
  about->exec();
}


//--------------------------------------------------------------------------

void mainForm::analysisIncidenceMatrix()
{
	int **m;
	int w, k;
	int i, j;
	m = net->matrix();
	w = net->placesCount();
	k = net->transitionsCount();
  
  QString elem;
  int maxLength = 0;
  int length;  
  for(i = 0; i < w; i++)
  {
    for(j = 0; j < k; j++)
    {
      elem = QString("%1").arg(m[i][j]);
      length = elem.length();
      if(length > maxLength) maxLength = length;
    }
  }
  maxLength++;
	
	QString temp = "------------------------------------------------------------\n";
  temp += "Incidence matrix\n"; 
  temp += "------------------------------------------------------------\n";
	for(i = 0; i < w; i++)
	{
		for(j = 0; j < k; j++)
		{
			elem = QString("%1").arg(m[i][j]);
      while(elem.length() < maxLength) elem = " " + elem;
      temp += elem;
			temp += "  ";
		}
		temp += "\n";
	}
	temp += "------------------------------------------------------------\n";
  
	reportTextEdit->append(temp);
}


//--------------------------------------------------------------------------

void mainForm::analysisInputMatrix()
{
  int **m;
  int w, k;
  int i, j;
  m = net->inMatrix();
  w = net->placesCount();
  k = net->transitionsCount();
  
  QString elem;
  int maxLength = 0;
  int length;  
  for(i = 0; i < w; i++)
  {
    for(j = 0; j < k; j++)
    {
      elem = QString("%1").arg(m[i][j]);
      length = elem.length();
      if(length > maxLength) maxLength = length;
    }
  }
  maxLength++;
  
  QString temp = "------------------------------------------------------------\n";
  temp += "Input matrix\n"; 
  temp += "------------------------------------------------------------\n";
  for(i = 0; i < w; i++)
  {
    for(j = 0; j < k; j++)
    {
      elem = QString("%1").arg(m[i][j]);
      while(elem.length() < maxLength) elem = " " + elem;
      temp += elem;
      temp += "  ";
    }
    temp += "\n";
  }
  temp += "------------------------------------------------------------\n";
  
  reportTextEdit->append(temp);
}


//--------------------------------------------------------------------------

void mainForm::analysisOutputMatrix()
{
  int **m;
  int w, k;
  int i, j;
  m = net->outMatrix();
  w = net->placesCount();
  k = net->transitionsCount();
  
  QString elem;
  int maxLength = 0;
  int length;  
  for(i = 0; i < w; i++)
  {
    for(j = 0; j < k; j++)
    {
      elem = QString("%1").arg(m[i][j]);
      length = elem.length();
      if(length > maxLength) maxLength = length;
    }
  }
  maxLength++;
  
  QString temp = "------------------------------------------------------------\n";
  temp += "Output matrix\n"; 
  temp += "------------------------------------------------------------\n";
  for(i = 0; i < w; i++)
  {
    for(j = 0; j < k; j++)
    {
      elem = QString("%1").arg(m[i][j]);
      while(elem.length() < maxLength) elem = " " + elem;
      temp += elem;
      temp += "  ";
    }
    temp += "\n";
  }
  temp += "------------------------------------------------------------\n";
  
  reportTextEdit->append(temp);
}


//--------------------------------------------------------------------------

void mainForm::analysisPlaceInvariants()
{
  int **m;
  int w, k;
  int i, j;
  m = net->matrix();
  w = net->placesCount();
  k = net->transitionsCount();
  
  CMatrix *matrix = new CMatrix(k, w);
  
  for(i = 0; i < w; i++)
  {
    for(j = 0; j < k; j++)
    {
      matrix->data[j][i] = m[i][j];
    }
  }
  
  matrix->solve();
  
  QString temp = "------------------------------------------------------------\n";
  temp += "Place invariants\n"; 
  temp += "------------------------------------------------------------\n";
  temp += matrix->invariants();
  temp += "------------------------------------------------------------\n";
  
  reportTextEdit->append(temp);
}


//--------------------------------------------------------------------------

void mainForm::analysisTransitionInvariants()
{
  int **m;
  int w, k;
  int i, j;
  m = net->matrix();
  w = net->placesCount();
  k = net->transitionsCount();
  
  CMatrix *matrix = new CMatrix(w, k);
  
  for(i = 0; i < w; i++)
  {
    for(j = 0; j < k; j++)
    {
      matrix->data[i][j] = m[i][j];
    }
  }
  
  matrix->solve();
  
  QString temp = "------------------------------------------------------------\n";
  temp += "Transition invariants\n"; 
  temp += "------------------------------------------------------------\n";
  temp += matrix->invariants();
  temp += "------------------------------------------------------------\n";
  
  reportTextEdit->append(temp);
}


//--------------------------------------------------------------------------

void mainForm::analysisReachabilityGraph()
{
	CReachGraph* graph;
  graph = new CReachGraph(net);
	
	graph->usePriorities(!(options->graphIgnorePriorities));
	graph->useTime(!(options->graphIgnoreTime));
	
	if(options->limitGraphSize)
	{
		graph->setMaxStateNumber(options->graphSizeLimit);
	}
	else
	{
		graph->setMaxStateNumber(10000);
	}
	graph->createGraph();
  
  QString temp = "------------------------------------------------------------\n";
  temp += "Reachability graph\n"; 
  temp += "------------------------------------------------------------\n";
  temp += graph->graphDescription();
  temp += "------------------------------------------------------------\n";
  reportTextEdit->append(temp);
	delete graph;
}


//--------------------------------------------------------------------------

void mainForm::analysisCoverabilityGraph()
{
  CReachGraph* graph;
  graph = new CCoverGraph(net);
  
  graph->usePriorities(!(options->graphIgnorePriorities));
  graph->useTime(!(options->graphIgnoreTime));
  
  if(options->limitGraphSize)
  {
    graph->setMaxStateNumber(options->graphSizeLimit);
  }
  else
  {
    graph->setMaxStateNumber(10000);
  }
  graph->createGraph();
  QString temp = "------------------------------------------------------------\n";
  temp += "Coverability Graph:\n"; 
  temp += "------------------------------------------------------------\n";
  temp += graph->graphDescription();
  temp += "------------------------------------------------------------\n";
  reportTextEdit->append(temp);
  delete graph;
}


//--------------------------------------------------------------------------

void mainForm::disactivateEditing()
{
	fileNewAction->setEnabled(false);
  fileOpenAction->setEnabled(false);
  fileSaveAction->setEnabled(false);
  fileSaveAsAction->setEnabled(false);
  fileSaveReportAction->setEnabled(false);
  fileExportAction->setEnabled(false);
  fileExitAction->setEnabled(false);
  editCopyAction->setEnabled(false);
  editCutAction->setEnabled(false);
  editPasteAction->setEnabled(false);
  editSelectAllAction->setEnabled(false);
  editDeleteAction->setEnabled(false);
  editPreferencesAction->setEnabled(false);
  netArcAction->setEnabled(false);
	netPlaceAction->setEnabled(false);
	netTransitionAction->setEnabled(false);
	netSelectAction->setEnabled(false);
  analysisIncidenceMatrixAction->setEnabled(false);
  analysisReachabilityGraphAction->setEnabled(false);
  alingmentLeftAction->setEnabled(false);
  alingmentRightAction->setEnabled(false);
  alingmentTopAction->setEnabled(false);
  alingmentBottomAction->setEnabled(false);
  alingmentHorizontalSpreadAction->setEnabled(false);
  alingmentVerticalSpreadAction->setEnabled(false);
}


//--------------------------------------------------------------------------

void mainForm::activateEditing()
{
	fileNewAction->setEnabled(true);
  fileOpenAction->setEnabled(true);
  fileSaveAction->setEnabled(true);
  fileSaveAsAction->setEnabled(true);
  fileSaveReportAction->setEnabled(true);
  fileExportAction->setEnabled(true);
  fileExitAction->setEnabled(true);
  //editCopyAction->setEnabled(true);
  //editCutAction->setEnabled(true);
  //editPasteAction->setEnabled(true);
  editSelectAllAction->setEnabled(true);
  editDeleteAction->setEnabled(true);
  editPreferencesAction->setEnabled(true);
  netArcAction->setEnabled(true);
  netPlaceAction->setEnabled(true);
  netTransitionAction->setEnabled(true);
  netSelectAction->setEnabled(true);
  analysisIncidenceMatrixAction->setEnabled(true);
  analysisReachabilityGraphAction->setEnabled(true);
  alingmentLeftAction->setEnabled(true);
  alingmentRightAction->setEnabled(true);
  alingmentTopAction->setEnabled(true);
  alingmentBottomAction->setEnabled(true);
  //alingmentHorizontalSpreadAction->setEnabled(true);
  //alingmentVerticalSpreadAction->setEnabled(true);
}


//--------------------------------------------------------------------------
/** Functions serves the Preferences Dialog. */

void mainForm::editPreferences()
{
	optionsDialog *dialog = new optionsDialog();
	
	// read options
	dialog->placeWidthSpinBox->setValue(options->placeWidth);
	dialog->placeHeightSpinBox->setValue(options->placeHeight);
	dialog->transWidthSpinBox->setValue(options->transitionWidth);
	dialog->transHeightSpinBox->setValue(options->transitionHeight);
	dialog->lineWidthSpinBox->setValue(options->lineWidth);
	dialog->arrowWidthSpinBox->setValue(options->arrowheadWidth);
	dialog->arrowHeightSpinBox->setValue(options->arrowheadHeight);
	
	dialog->timeCheckBox->setChecked(options->ignoreTime);
	dialog->prioritiesCheckBox->setChecked(options->ignorePriorities);
	
	dialog->prioritiesGraphCheckBox->setChecked(options->graphIgnorePriorities);
	dialog->timeGraphCheckBox->setChecked(options->graphIgnoreTime);
	dialog->limitCheckBox->setChecked(options->limitGraphSize);
	dialog->nodesSpinBox->setValue(options->graphSizeLimit);
	
	dialog->sizeComboBox->setCurrentItem(options->xfigSize);
	dialog->orientationComboBox->setCurrentItem(options->xfigOrientation);
	dialog->xfigLineWidthSpinBox->setValue(options->xfigLineWidth);
	dialog->fillShapesCheckBox->setChecked(options->xfigFillShapes);
	
	dialog->canvasWidthSpinBox->setValue(options->svgCanvasWidth);
	dialog->canvasHeightSpinBox->setValue(options->svgCanvasHeight);
	dialog->xSpinBox->setValue(options->svgX);
	dialog->ySpinBox->setValue(options->svgY);
	dialog->viewWidthSpinBox->setValue(options->svgViewWidth);
	dialog->viewHeightSpinBox->setValue(options->svgViewHeight);
	dialog->stylesheetLineEdit->setText(options->svgStylesheet);
	dialog->scriptLineEdit->setText(options->svgScript);
	dialog->scriptTypeComboBox->setCurrentText(options->svgScriptType);
	
	// write options
	if(dialog->exec())
	{
		options->placeWidth = dialog->placeWidthSpinBox->value();
		options->placeHeight = dialog->placeHeightSpinBox->value();
		options->transitionWidth = dialog->transWidthSpinBox->value();
		options->transitionHeight = dialog->transHeightSpinBox->value();
		options->lineWidth = dialog->lineWidthSpinBox->value();
		options->arrowheadWidth = dialog->arrowWidthSpinBox->value();
		options->arrowheadHeight = dialog->arrowHeightSpinBox->value();
		
		options->ignoreTime = dialog->timeCheckBox->isChecked();
		options->ignorePriorities = dialog->prioritiesCheckBox->isChecked();
    net->setSimulationParameters(!options->ignorePriorities, !options->ignoreTime);
		netView->setSimulationParameters(!options->ignorePriorities, !options->ignoreTime);
  
		options->graphIgnorePriorities = dialog->prioritiesGraphCheckBox->isChecked();
		options->graphIgnoreTime = dialog->timeGraphCheckBox->isChecked();
		options->limitGraphSize = dialog->limitCheckBox->isChecked();
		options->graphSizeLimit = dialog->nodesSpinBox->value();
		
		options->xfigSize = dialog->sizeComboBox->currentItem();
		options->xfigOrientation = dialog->orientationComboBox->currentItem();
		options->xfigLineWidth = dialog->xfigLineWidthSpinBox->value();
		options->xfigFillShapes = dialog->fillShapesCheckBox->isChecked();
		
		options->svgCanvasWidth = dialog->canvasWidthSpinBox->value();
		options->svgCanvasHeight = dialog->canvasHeightSpinBox->value();
		options->svgX = dialog->xSpinBox->value();
		options->svgY = dialog->ySpinBox->value();
		options->svgViewWidth = dialog->viewWidthSpinBox->value();
		options->svgViewHeight = dialog->viewHeightSpinBox->value();
		options->svgStylesheet = dialog->stylesheetLineEdit->text();
		options->svgScript = dialog->scriptLineEdit->text();
		options->svgScriptType = dialog->scriptTypeComboBox->currentText();
    
    // sets new options for the NetView
    netView->setPlaceWidth(options->placeWidth);
    netView->setPlaceHeight(options->placeHeight);
    netView->setTransitionWidth(options->transitionWidth);
    netView->setTransitionHeight(options->transitionHeight);
    netView->setLineWidth(options->lineWidth);
    netView->setArrowheadWidth(options->arrowheadWidth);
    netView->setArrowheadHeight(options->arrowheadHeight);   
    netView->updateElements(); 
	}
}


//--------------------------------------------------------------------------
/** Writes settings to the settings file. */

void mainForm::writeSettings()
{
	QSettings settings(QSettings::Ini);			
	settings.insertSearchPath(QSettings::Unix, qApp->applicationDirPath());
	settings.beginGroup("/ape");
	
	settings.writeEntry("/Editor/placeWidth", options->placeWidth);
	settings.writeEntry("/Editor/placeHeight", options->placeHeight);
	settings.writeEntry("/Editor/transitionWidth", options->transitionWidth);
	settings.writeEntry("/Editor/transitionHeight", options->transitionHeight);
	settings.writeEntry("/Editor/lineWidth", options->lineWidth);
	settings.writeEntry("/Editor/arrowheadWidth", options->arrowheadWidth);
	settings.writeEntry("/Editor/arrowheadHeight", options->arrowheadHeight);
	
	settings.writeEntry("/Simulator/ignoreTime", options->ignoreTime);
	settings.writeEntry("/Simulator/ignorePriorities", options->ignorePriorities);
	
	settings.writeEntry("/Graphs/ignoreTime", options->graphIgnorePriorities);
	settings.writeEntry("/Graphs/ignorePriorities", options->graphIgnoreTime);
	settings.writeEntry("/Graphs/limitGraphSize", options->limitGraphSize);
	settings.writeEntry("/Graphs/graphSizeLimit", options->graphSizeLimit);
	
  settings.writeEntry("/XfigExport/size", options->xfigSize);
	settings.writeEntry("/XfigExport/orientation", options->xfigOrientation);
	settings.writeEntry("/XfigExport/lineWidth", options->xfigLineWidth);
	settings.writeEntry("/XfigExport/fillShapes", options->xfigFillShapes);
	
	settings.writeEntry("/SVGExport/canvasWidth", options->svgCanvasWidth);
	settings.writeEntry("/SVGExport/canvasHeight", options->svgCanvasHeight);
	settings.writeEntry("/SVGExport/viewX", options->svgX);
	settings.writeEntry("/SVGExport/viewY", options->svgY);
	settings.writeEntry("/SVGExport/viewWidth", options->svgViewWidth);
	settings.writeEntry("/SVGExport/viewHeight", options->svgViewHeight);
	settings.writeEntry("/SVGExport/stylesheet", options->svgStylesheet);
	settings.writeEntry("/SVGExport/script", options->svgScript);
	settings.writeEntry("/SVGExport/scriptType", options->svgScriptType);

	settings.endGroup();
}


//--------------------------------------------------------------------------
/** Reads settings from the settings file. */

void mainForm::readSettings()
{
	QSettings settings(QSettings::Ini);			
	settings.insertSearchPath(QSettings::Unix, qApp->applicationDirPath());
	settings.beginGroup("/ape");
	
	options->placeWidth = settings.readNumEntry("/Editor/placeWidth", 50); 
	options->placeHeight = settings.readNumEntry("/Editor/placeHeight", 35);
	options->transitionWidth = settings.readNumEntry("/Editor/transitionWidth", 45);
	options->transitionHeight = settings.readNumEntry("/Editor/transitionHeight", 25);
	options->lineWidth = settings.readNumEntry("/Editor/lineWidth", 1);
	options->arrowheadWidth = settings.readNumEntry("/Editor/arrowheadWidth", 10);
	options->arrowheadHeight = settings.readNumEntry("/Editor/arrowheadHeight", 14);
	
	options->ignoreTime = settings.readBoolEntry("/Simulator/ignoreTime", true);
	options->ignorePriorities = settings.readBoolEntry("/Simulator/ignorePriorities", true);
	
	options->graphIgnorePriorities = settings.readBoolEntry("/Graphs/ignoreTime", true);
	options->graphIgnoreTime = settings.readBoolEntry("/Graphs/ignorePriorities", true);
	options->limitGraphSize = settings.readBoolEntry("/Graphs/limitGraphSize", true);
  options->graphSizeLimit = settings.readNumEntry("/Graphs/graphSizeLimit", 1000);
	
  options->xfigSize = settings.readNumEntry("/XfigExport/size", 0);
	options->xfigOrientation = settings.readNumEntry("/XfigExport/orientation", 0);
	options->xfigLineWidth = settings.readNumEntry("/XfigExport/lineWidth", 1);
	options->xfigFillShapes = settings.readBoolEntry("/XfigExport/fillShapes", false);

	options->svgCanvasWidth = settings.readNumEntry("/SVGExport/canvasWidth", 1512);
	options->svgCanvasHeight = settings.readNumEntry("/SVGExport/canvasHeight", 1512);
	options->svgX = settings.readNumEntry("/SVGExport/viewX", 0);
	options->svgY = settings.readNumEntry("/SVGExport/viewY", 0);
	options->svgViewWidth = settings.readNumEntry("/SVGExport/viewWidth", 1512);
	options->svgViewHeight = settings.readNumEntry("/SVGExport/viewHeight", 1512);
	options->svgStylesheet = settings.readEntry("/SVGExport/stylesheet", "style.css");
	options->svgScript = settings.readEntry("/SVGExport/script", "engine.js");
	options->svgScriptType = settings.readEntry("/SVGExport/scriptType", "(Empty)");
	
	settings.endGroup();
}


//--------------------------------------------------------------------------

void mainForm::closeEvent( QCloseEvent *e )
{
	writeSettings();
	e->accept();
}


//--------------------------------------------------------------------------

void mainForm::createPropertyEditor()
{
	QTable *table;
	
	/* Create a properties editor and add it to the main form. */
	propertiesEditor = new CPropertiesEditor(this, "Properties Editor");
	moveDockWindow(propertiesEditor, Left);
	
	// Create property editor table for pages
	propertiesEditor->addTable("Property", "Value",3);
	table = propertiesEditor->table(0);
	table->setText(0,0,"Name");
	table->setText(1,0,"Width");
	table->setText(2,0,"Height");
	for(int i = 0; i < 3; i++)
  {
    table->setItem(i, 1, new CTableItem(table, ""));
  }
	
	// Create property editor table for places
	propertiesEditor->addTable("Property", "Value",4);
	table = propertiesEditor->table(1);
	table->setText(0,0,"Name");
	table->setText(1,0,"Capacity");
	table->setText(2,0,"Marking");
	table->setText(3,0,"ID");
	for(int i = 0; i < 4; i++)
  {
    table->setItem(i, 1, new CTableItem(table, ""));
  }
	
	// Create property editor table for transitions
	propertiesEditor->addTable("Property", "Value",4);
	table = propertiesEditor->table(2);
	table->setText(0,0,"Name");
	table->setText(1,0,"Priority");
	table->setText(2,0,"Time");
	table->setText(3,0,"ID");
	for(int i = 0; i < 4; i++)
  {
    table->setItem(i, 1, new CTableItem(table, ""));
  }
	
	// Create property editor table for arcs
	propertiesEditor->addTable("Property", "Value",1);
	table = propertiesEditor->table(3);
	table->setText(0,0,"Weight");
	
	// Create time label
	timeLabel = new QLabel(this);
	timeLabel->setText("time = 0");
	timeLabel->setEnabled(true);
	QStatusBar* statusBar = this->statusBar();
	statusBar->addWidget(timeLabel, 0, true);
}


//--------------------------------------------------------------------------

void mainForm::createConnections()
{
	// netView modes
	connect(netSelectAction, SIGNAL(activated()),
					netView, SLOT(setSelectMode()));
	connect(netPlaceAction, SIGNAL(activated()),
					netView, SLOT(setAddPlaceMode()));
	connect(netTransitionAction, SIGNAL(activated()),
					netView, SLOT(setAddTransMode()));
	connect(netArcAction, SIGNAL(activated()),
					netView, SLOT(setAddArcMode()));
	
	// alingment
	connect(alingmentLeftAction, SIGNAL(activated()),
					netView, SLOT(alingmentLeft()));
	connect(alingmentRightAction, SIGNAL(activated()),
					netView, SLOT(alingmentRight()));
	connect(alingmentTopAction, SIGNAL(activated()),
					netView, SLOT(alingmentTop()));
	connect(alingmentBottomAction, SIGNAL(activated()),
					netView, SLOT(alingmentBottom()));
	connect(alingmentHorizontalSpreadAction, SIGNAL(activated()),
					netView, SLOT(alingmentHorizontalSpread()));
	connect(alingmentVerticalSpreadAction, SIGNAL(activated()),
					netView, SLOT(alingmentVerticalSpread()));
	
	// simulation
	connect(simulationInteractiveRunAction, SIGNAL(activated()),
					netView, SLOT(setInteractiveRunMode()));
	connect(simulationAutomaticRunAction, SIGNAL(activated()),
					netView, SLOT(setAutomaticRunMode()));
	connect(simulationStopAction, SIGNAL(activated()),
					netView, SLOT(stopSimulation()));
	connect(simulationInitialMarkingAction, SIGNAL(activated()),
					netView, SLOT(resetInitialMarking()));
	
	// edition
	connect(editUndoAction, SIGNAL(activated()), netView, SLOT(undo()));
	connect(editRedoAction, SIGNAL(activated()), netView, SLOT(redo()));
	connect(editCutAction, SIGNAL(activated()), netView, SLOT(cut()));
	connect(editCopyAction, SIGNAL(activated()), netView, SLOT(copy()));
	connect(editPasteAction, SIGNAL(activated()), netView, SLOT(paste()));
	connect(editDeleteAction, SIGNAL(activated()), netView, SLOT(remove()));
	connect(editSelectAllAction, SIGNAL(activated()),netView, SLOT(selectAll()));
	
	connect(netView, SIGNAL(canvasSelected(CNetView*)),
					propertiesEditor, SLOT(canvasSelected(CNetView*)));
	connect(netView, SIGNAL(placeSelected(CEllipseWithLabels*)),
					propertiesEditor, SLOT(placeSelected(CEllipseWithLabels*)));
	connect(netView, SIGNAL(transSelected(CRectangleWithLabels*)),
					propertiesEditor, SLOT(transSelected(CRectangleWithLabels*)));
	connect(netView, SIGNAL(arcSelected(CLineWithLabels*)),
					propertiesEditor, SLOT(arcSelected(CLineWithLabels*)));
	
	connect(propertiesEditor->table(0), SIGNAL(valueChanged(int,int)),
					propertiesEditor, SLOT(propertyChanged(int,int)));
	connect(propertiesEditor->table(1), SIGNAL(valueChanged(int,int)),
					propertiesEditor, SLOT(propertyChanged(int,int)));
	connect(propertiesEditor->table(2), SIGNAL(valueChanged(int,int)),
					propertiesEditor, SLOT(propertyChanged(int,int)));
	connect(propertiesEditor->table(3), SIGNAL(valueChanged(int,int)),
					propertiesEditor, SLOT(propertyChanged(int,int)));
	
	connect(propertiesEditor, SIGNAL(setCanvasProperties(QString,int,int)),
					netView, SLOT(setCanvasProperties(QString,int,int)));
	connect(propertiesEditor, SIGNAL(setTransProperties(QString,int,int,int)),
					netView, SLOT(setTransProperties(QString,int,int,int)));
	connect(propertiesEditor, SIGNAL(setPlaceProperties(QString,int,int,int)),
					netView, SLOT(setPlaceProperties(QString,int,int,int)));
	connect(propertiesEditor, SIGNAL(setArcProperties(int)),
					netView, SLOT(setArcProperties(int)));
	
	connect(netView, SIGNAL(updateClock(int)),
					this, SLOT(updateClock(int)));
}


//--------------------------------------------------------------------------

void mainForm::createWidgets()
{
  setCentralWidget(new QWidget(this, "qt_central_widget"));
	formLayout = new QGridLayout(centralWidget(), 1, 1, 1, 1, "formLayout");
	widgetStack = new QWidgetStack(centralWidget(), "tabWidget");	
		
	canvas = new QCanvas(widgetStack);
	canvas->resize(1500, 1500);
	netView = new CNetView(widgetStack, canvas, net);	
	void setMode(CNetViewMode m);
	netView->setPlaceWidth(options->placeWidth);
	netView->setPlaceHeight(options->placeHeight);
	netView->setTransitionWidth(options->transitionWidth);
	netView->setTransitionHeight(options->transitionHeight);
	netView->setLineWidth(options->lineWidth);
	netView->setArrowheadWidth(options->arrowheadWidth);
	netView->setArrowheadHeight(options->arrowheadHeight);
	netView->setName("Net");
	
	formLayout->addWidget(widgetStack, 0, 0);
	
	/* Create a report editor and add it to the main form. */
	reportEditor = new CReportEditor(this, "Report");
	moveDockWindow(reportEditor, Right);	
	reportTextEdit = reportEditor->editor();
	
}


//--------------------------------------------------------------------------

void mainForm::updateClock(int time)
{
  QString timeValue("time = ");
  timeValue = timeValue + QString::number(time);
	timeLabel->setText(timeValue);
}


//--------------------------------------------------------------------------

void mainForm::clearReport()
{
  reportTextEdit->clear();
}








