/***************************************************************************
    copyright            : (C) 2004 - 2005 by Marcin Szpyrka
    email                : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CARC_H
#define CARC_H

#include "cplace.h"
#include "ctransition.h"

/**
CArcDir type contains constants that denote two possible
arc direction:  adPT and adTP.
*/

enum CArcDir {adPT = 0, adTP = 1};

/**
CArc class objects represent arcs of a net.
Object attributes: arc tag, arc weight, arc direction
and pointer to beginning and ending nodes.
*/

class CArc
{
protected:
  int          arcTag;       // arc identifier
  int          arcWeight;
  CPlace*      placeNode;
  CTransition* transNode;
  CArcDir      arcDirection;

public:
  /**
    Constructs a CArc from a place to a transition.
    \param tag the arc tag,
    \param p a pointer to the place node of the arc,
    \param t a pointer to the transition node of the arc,
    \param w the arc weight,
  */
  CArc(int tag, CPlace* p, CTransition* t, int w = 1) :
      arcTag(tag),
      arcWeight(w),
      placeNode(p),
      transNode(t),
      arcDirection(adPT)
  {}

  /** Constructs a CArc from a transition to a place.
    This is an overloaded constructor, provided for convenience.
    It behaves essentially like the above function. */
  CArc(int tag, CTransition* t, CPlace* p, int w = 1) :
      arcTag(tag),
      arcWeight(w),
      placeNode(p),
      transNode(t),
      arcDirection(adTP)
  {}

  /** Destroys the arc. */
  ~CArc(){}

  /** Returns the direction of the arc. */
  CArcDir direction() const { return arcDirection; }

  /** Returns the tag of the arc. */
  int tag() const { return arcTag; }

  /** Returns the weight of the arc. */
  int weight() const { return arcWeight; }

  /** Sets the arc weight to \a w.
      If the \a w < 0, the arc weight = 0 is set. */
  void setWeight(int w){ arcWeight = w > 0 ? w : 0; }

  /** Returns a pointer to the place node of the arc. */
  CPlace* place() const { return placeNode; }

  /** Returns a pointer to the transition node of the arc. */
  CTransition* transition() const { return transNode; }
};

#endif








