/***************************************************************************
    copyright            : (C) 2004 - 2005 by Marcin Szpyrka
    email                : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CPLACE_H
#define CPLACE_H

#include "cnode.h"

/**
CPlace class objects represent places of a net.
Object attributes: place tag, place name, place capacity, current marking
and initial marking.
*/

class CPlace : public CNode
{
protected:
  int    placeCapacity;  // 0 denotes infinity
  int    currentMarking;
  int    initMarking;

public:
  /**
    Constructs a CPlace. The place marking is set to
    the initial marking.
    \param tag the place tag,
    \param name the place name,
    \param m the place initial marking,
    \param c the place capacity.
  */
  CPlace(int tag, QString name, int m = 0,int c = 0) :
      CNode(tag, name),
      placeCapacity(c),
      currentMarking(m),
      initMarking(m)
  {}

  /** Destroys the place.  */
  ~CPlace(){}

  /** Returns the capacity of the place.
    0 denotes the infinite capacity. */
  int capacity() const { return placeCapacity; }

  /** Sets the place capacity to \a c.
  If \a c < 0, the place capacity = 0 (infinite) is set. */
  void setCapacity(int c){ placeCapacity = c > 0 ? c : 0; }

  /** Returns the marking of the place.  */
  int marking() const { return currentMarking; }

  void setMarking(int m);

  /** Returns the initial marking of the place.  */
  int initialMarking() const { return initMarking; }

  void setInitialMarking(int m);
};

#endif

