/***************************************************************************
    copyright            : (C) 2004 - 2005 by Marcin Szpyrka
    email                : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CTRANSITION_H
#define CTRANSITION_H

#include "cnode.h"

/**
CTransition class objects represent transitions of a net.
Object attributes: transition tag, transition name
and transition priority.
*/

class CTransition : public CNode
{
protected:
  int transPriority;  // 0 denotes the lowest priority
  int transTime;
  int waitTime;

public:
  /**
    Constructs a CTransition.
    \param tag the transition tag,
    \param name the transition name,
    \param pr the transition priority.
  */
  CTransition(int tag, QString name, int pr = 0, int t = 0) :
      CNode(tag, name),
      transPriority(pr),
      transTime(t),
      waitTime(-1)
  {}

  /** Destroys the transition.  */
  virtual ~CTransition(){}

  /** Returns the priority of the transition. */
  int priority() const { return transPriority; }

  /** Returns the time delay of the transition. */
  int time() const { return transTime; }

  /** Sets the transition priority to \a pr.
      If \a pr < 0, the transition priority = 0 is set. */
  void setPriority(int pr){ transPriority = pr > 0 ? pr : 0; }

  /** Sets the transition time delay to \a t.
  If \a t < 0, the transition time delay = 0 is set. */
  void setTime(int t){ transTime = t > 0 ? t : 0; }

  /** Returns the waiting time i.e., how long an active transition
  has to wait to be fireable. */
  int waitingTime() const { return waitTime; }
  
  /** Sets the waiting time i.e., how long an active transition
  has to wait to be fireable. */
  void setWaitingTime(int t){ waitTime = t; }

};

#endif
