/***************************************************************************
    copyright   : (C) 2004 - 2005 by 
    email       : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cxfigexport.h"

const int CXfigExport::fsize = 17;
const int CXfigExport::scale = 15;

//--------------------------------------------------------------------------
/**
Creates new xfig export generator:
  \param f the output file,
  \param width lines' thickness,
  \param orientation paper orientation,
  \param papersize paper size,
  \param fill fill or not figures.
*/

CXfigExport::CXfigExport(QFile&f, int width, int orientation, int papersize, bool fill) : file(f)
{
	this->orientation = orientation;
	this->fill = fill;
	this->papersize = papersize;
	this->width = width;
}


//--------------------------------------------------------------------------
/** Exports net contained by \a canvas. 
After calling this method the CXfigExport object is no more valid 
and should be destroyed. */

void CXfigExport::exportNet(QCanvas* canvas)
{
	stream = new QTextStream(&file);
	(*stream) << "#FIG 3.2\n";
	
	if (orientation == Portrait) (*stream) << "Portrait\n";
	else (*stream) << "Landscape\n";
		
	(*stream) << "Center\nInches\n";
	
	switch (papersize) {
		case Letter:  (*stream) << "Letter\n";  break;
		case Legal:   (*stream) << "Legal\n";   break;
		case Ledger:  (*stream) << "Ledger\n";  break;
		case Tabloid: (*stream) << "Tabloid\n"; break;
		case A:		    (*stream) << "A\n";  			break;
		case B: 			(*stream) << "B\n"; 			break;
		case C: 			(*stream) << "C\n"; 			break;
		case D: 			(*stream) << "D\n"; 			break;
		case E: 			(*stream) << "E\n"; 			break;
		case A4: 			(*stream) << "A4\n";			break;
		case A3:			(*stream) << "A3\n";			break;
		case A2:			(*stream) << "A2\n";			break;
		case A1:			(*stream) << "A1\n";			break;
		case A0:			(*stream) << "A0\n";			break;
		case B5:			(*stream) << "B5\n";
	}
	
	(*stream) << "100.00\nSingle\n7\n";	
	(*stream) << "1200 2\n";
	
	QCanvasItemList items = canvas->allItems();
	QCanvasItemList::iterator it;
	
	it = items.begin();
	for (; it != items.end(); it++)
	{
		if((*it)->rtti() == 10005)  // CEllipseWithLabels
    {  
			writeEllipseWithLabels((CEllipseWithLabels*)(*it));
    }  
		else if((*it)->rtti() == 10004)  // CRectangleWithLabels
    {  
			writeRectangleWithLabels((CRectangleWithLabels*)(*it));
    }   
		else if (((*it)->rtti() == 10013) && (((CSegment *)(*it))->tag() == 0)) // CSegment
		{
			CSegment *s;
			if((s = (CSegment*)(*it)))
			{
				CLineWithLabels *arc;
				if ((arc = (CLineWithLabels*)(s->owner())))
        {    
					writeLineWithLabels(arc);
        }  
			}
		}
	}
	
	delete stream;
	file.close();
}


//--------------------------------------------------------------------------
/** Writes a label containing \a txt at position defined by \a x and \a y. */

void CXfigExport::writeLabel(QString txt, int x, int y)
{
	(*stream) << "4 0 0 50 -1 ";
	(*stream) << "16 "; // font (16 = helvetica)
	(*stream) << QString("%1 ").arg(fsize); // font size
	(*stream) << "0.0000 4 500 2000 ";	
	(*stream) << QString("%1 %1 ").arg((x+3)*scale).arg((y+fsize)*scale); // coordinates
	(*stream) << txt;
	(*stream) << "\\001\n";
}


//--------------------------------------------------------------------------
/** Writes an ellipse pointed by \a cewl togather with associated labels. */

void CXfigExport::writeEllipseWithLabels(CEllipseWithLabels* cewl)
{
	(*stream) << "1 "; // ellipse
	(*stream) << "2 "; // ??? definicja diametralna
	(*stream) << "0 "; // line style
	(*stream) << QString("%1 ").arg(width); // thickness
	(*stream) << "0 7 "; // colour 
	(*stream) << "51 "; // depth
	(*stream) << "-1 "; // 
	
	// filling
	if(fill) (*stream) <<  "20 ";
	else (*stream) <<  "-1 ";
		
	(*stream) << "0 "; // style
	(*stream) << "1 "; // direction
	(*stream) << "0 "; // angle
		
	// center
	(*stream) << QString("%1 %1 ").arg((cewl->x()+cewl->width()/2)*scale)
    .arg((cewl->y()+cewl->height()/2)*scale);	
	// half of the width
	(*stream) << QString("%1 %1 ").arg(cewl->width()/2*scale)
    .arg(cewl->height()/2*scale);	
	// left top corner
	(*stream) << QString("%1 %1 ").arg(cewl->x()*scale)
    .arg(cewl->y()*scale);	
	// right bottom corner
	(*stream) << QString("%1 %1\n").arg((cewl->x()+cewl->width())*scale)
    .arg((cewl->y()+cewl->height())*scale);	
	// label	
	writeLabel(cewl->label(0)->text(), (int)cewl->label(0)->x(), 
             (int)cewl->label(0)->y());
	
	CPlace *placePtr = (CPlace*)cewl->modelPointer();	
	switch (placePtr->initialMarking())
	{
		case 0: break;
		case 1:
			writeDot((int)(cewl->x() + (cewl->width() - 6) / 2), 
               (int)(cewl->y() + (cewl->height() - 6) / 2));
			break;
		case 2:
			writeDot((int)(cewl->x() + (cewl->width() - 6) / 3), 
               (int)(cewl->y() + (cewl->height() - 6) / 2));
			writeDot((int)(cewl->x() + 2 * (cewl->width() - 6) / 3),	
               (int)(cewl->y() + (cewl->height() - 6) / 2));
			break;
		case 3:
			writeDot((int)(cewl->x() + (cewl->width() - 6) / 2), 
               (int)(cewl->y() + (cewl->height() - 6) / 3) - 1);
			writeDot((int)(cewl->x() + (cewl->width() - 6) / 3), 
               (int)(cewl->y() + 2 * (cewl->height() - 6) / 3) + 1);
			writeDot((int)(cewl->x() + 2 * (cewl->width() - 6) / 3), 
               (int)(cewl->y() + 2 * (cewl->height() - 6) / 3) + 1);
			break;
		case 4:
			writeDot((int)(cewl->x() + (cewl->width() - 6) / 3), 
               (int)(cewl->y() + (cewl->height() - 6) / 3) - 1);
			writeDot((int)(cewl->x() + 2 * (cewl->width() - 6) / 3), 
               (int)(cewl->y() + (cewl->height() - 6) / 3) - 1);
			writeDot((int)(cewl->x() + (cewl->width() - 6) / 3), 
               (int)(cewl->y() + 2 * (cewl->height() - 6) / 3) + 1);
			writeDot((int)(cewl->x() + 2 * (cewl->width() - 6) / 3), 
               (int)(cewl->y() + 2 * (cewl->height() - 6) / 3) + 1);
			break;
		default:
			QString dotsLabel;
			dotsLabel = QString::number(placePtr->initialMarking());
			
			(*stream) << "4 1 0 50 -1 ";
			(*stream) << "0 "; // font
			(*stream) << QString("%1 ").arg(18); // font size
			(*stream) << "0.0000 4 500 2000 ";
			
			// coordinates
			(*stream) << QString("%1 %1").arg((cewl->x()+cewl->width()/2)*scale)
        .arg((cewl->y()+cewl->height()/2+9-3)*scale);
			
			(*stream) << dotsLabel;
			(*stream) << "\\001\n";
	}
}


//--------------------------------------------------------------------------
/** Writes a marking dot at position defined by \a x and \a y. */

void CXfigExport::writeDot(int x, int y)
{
	x += 3;
	y += 3;

	(*stream) << "1 "; // ellipse
	(*stream) << "2 "; // ??? definicja diametralna
	(*stream) << "0 "; // line style
	(*stream) << "1 "; // thickness
	(*stream) << "0 7 "; // colours 
	(*stream) << "50 "; // depth
	(*stream) << "-1 "; // 
	(*stream) << "0 "; // filling
	(*stream) << "0 "; // style
	(*stream) << "1 "; // direction
	(*stream) << "0 "; // angle
		
	// center
	(*stream) << QString("%1 %1 ").arg(x*scale).arg(y*scale);	
	// half of the width
	(*stream) << QString("%1 %1 ").arg(3*scale).arg(3*scale);	
	// left top corner
	(*stream) << QString("%1 %1 ").arg((x-3)*scale).arg((y-3)*scale);	
	// right bottom corner
	(*stream) << QString("%1 %1\n").arg((x+3)*scale).arg((y+3)*scale);
}


//--------------------------------------------------------------------------
/** Writes a rectangle at position (\a x, \a y) with size \a w by \a h. */

void CXfigExport::writeRectangle(int x, int y, int w, int h)
{
	(*stream) << "2 2 0 ";
	(*stream) << QString("%1 ").arg(width);
	(*stream) << "0 7 51 -1 ";
	
	if (fill)	(*stream) <<  "20 ";
	else (*stream) <<  "-1 ";
	
	(*stream) << "0.000 0 0 -1 0 0 5\n\t ";
	(*stream) << QString("%1 %1 ").arg(x*scale).arg(y*scale);
	(*stream) << QString("%1 %1 ").arg((x+w)*scale).arg(y*scale);
	(*stream) << QString("%1 %1 ").arg((x+w)*scale).arg((y+h)*scale);
	(*stream) << QString("%1 %1 ").arg(x*scale).arg((y+h)*scale);
	(*stream) << QString("%1 %1\n").arg(x*scale).arg(y*scale);
}


//--------------------------------------------------------------------------
/** Writes a rectangle pointed by \a crwl togather with associated labels. */

void CXfigExport::writeRectangleWithLabels(CRectangleWithLabels* crwl)
{
	writeRectangle((int)crwl->x(), (int)crwl->y(), crwl->width(), crwl->height());
	
	// label
	writeLabel(crwl->label(0)->text(), (int)crwl->label(0)->x(), 
             (int)crwl->label(0)->y());
			
	if (crwl->labelsCount()>1 && crwl->label(1)->isVisible())
	{
		writeLabel(crwl->label(1)->text(), (int)crwl->label(1)->x(), 
               (int)crwl->label(1)->y());
		
		writeRectangle((int)crwl->label(1)->x(), (int)crwl->label(1)->y(), 
                   crwl->label(1)->width() - 1, crwl->label(1)->height() - 1);
	}
}


//--------------------------------------------------------------------------
/** Writes a polyline pointed by \a arc togather with associated labels. */

void CXfigExport::writeLineWithLabels(CLineWithLabels* arc)
{
	if (CArc* arcPtr = static_cast<CArc* >(arc->modelPointer()))
	{
		(*stream) << "2 1 ";                    // id
		(*stream) << "0 ";                      // style
		(*stream) << QString("%1 ").arg(width); // thickness
		(*stream) << "0 ";                      // pen colour
		(*stream) << "7 ";                      // brush colour
		(*stream) << "50 ";                     // depth
		(*stream) << "-1 ";                     // 
		(*stream) << "-1 ";                     // filling
		(*stream) << "0 0 0 ";                  // style
		(*stream) << "-1 ";                     // angle
		
		// arrowhead
		if (arcPtr->direction() == adTP) (*stream) << "0 1 ";
		else	(*stream) << "1 0 ";
		
		(*stream) << QString("%1\n\t").arg(arc->breaksCount()+2); // liczba punktow lamania
		
		// arrowhead parameters
		(*stream) << "1 1 4.00 75.00 120.00\n\t";
    		
		// start point
		(*stream) << QString(" %1").arg(arc->breakpoint(-2).x()*scale);
		(*stream) << QString(" %1").arg(arc->breakpoint(-2).y()*scale);
		
		// break point
		for (int i=0; i<arc->breaksCount(); i++)
		{
			(*stream) << QString(" %1").arg(arc->breakpoint(i).x()*scale);
			(*stream) << QString(" %1").arg(arc->breakpoint(i).y()*scale);
		}
		
		// end point
		(*stream) << QString(" %1").arg(arc->breakpoint(-1).x()*scale);
		(*stream) << QString(" %1").arg(arc->breakpoint(-1).y()*scale);
		(*stream) << "\n";
		
		if (arc->labelsCount()>0 && arc->label(0)->isVisible())
			writeLabel(arc->label(0)->text(), (int)arc->label(0)->x(), (int)arc->label(0)->y());
	}
}
