/***************************************************************************
    copyright            : (C) 2004 - 2005 by Marcin Szpyrka
    email                : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cxmlreader.h"
#include "../view/cellipsewithlabels.h"
#include "../model/cnet.h"

//--------------------------------------------------------------------------
/** Constructs an XML reader with \a device and \a codec. */

CXMLReader::CXMLReader(CNet* n, CNetView* nv)
{
  net = n;
  netView = nv;
  netCanvas = netView->canvas();
  placeID = 1;
  transID = 1;
  currLine = 0;

  // node sizes
  plWidth  = netView->placeWidth();
  plHeight = netView->placeHeight();
  trWidth  = netView->transitionWidth();
  trHeight = netView->transitionHeight();
  linWidth = netView->lineWidth();
  ahWidth  = netView->arrowheadWidth();
  ahHeight = netView->arrowheadHeight();

}


//--------------------------------------------------------------------------
/** The reader calls this function when it has parsed a start element tag. */

bool CXMLReader::startElement(const QString &/*namespaceURI*/,
  const QString &/*localName*/, const QString &qName, const QXmlAttributes &attribs)
{
  if(qName == "net")
  {
    int x, y;

    x = attribs.value(1).toInt();
    y = attribs.value(2).toInt();
    netView->setName(attribs.value(0));
    netCanvas->resize(x,y);
  }

  if(qName == "place")
  {
    int m, c, x, y, nx, ny;

    m = attribs.value(1).toInt();
    c = attribs.value(2).toInt();
    x = attribs.value(3).toInt();
    y = attribs.value(4).toInt();
    nx = attribs.value(5).toInt();
    ny = attribs.value(6).toInt();
    CPlace *cp = new CPlace(placeID, attribs.value(0), m, c);
    net->addPlace(cp);

    // create the view place (ellipse)
    CEllipseWithLabels *eli
      = new CEllipseWithLabels(placeID, x, y, plWidth, plHeight, netCanvas);

    eli->setPen(QPen(Qt::black, linWidth));
    eli->setBrush(Qt::white);
    eli->setDots(m);
    eli->addLabel(0, nx, ny, attribs.value(0), false);
    eli->label(0)->setZ(3);
    eli->setModelPointer(cp);
    eli->setZ(1);
    eli->show();
    placeID++;
    //netCanvas->update();
    return true;
  }

  if(qName == "transition")
  {
    int p, t, x, y, nx, ny, ptx, pty;
    QString label2;

    label2 = attribs.value(1);
    label2 += "|";
    label2 += attribs.value(2);
    p = attribs.value(1).toInt();
    t = attribs.value(2).toInt();
    x = attribs.value(3).toInt();
    y = attribs.value(4).toInt();
    nx = attribs.value(5).toInt();
    ny = attribs.value(6).toInt();
    ptx = attribs.value(7).toInt();
    pty = attribs.value(8).toInt();
    CTransition *ct = new CTransition(transID, attribs.value(0), p);
    net->addTransition(ct);
    ct->setTime(t);

    // create the view transition (rectangle)
    CRectangleWithLabels *rec =
      new CRectangleWithLabels(transID, x, y, trWidth, trHeight, netCanvas);

    rec->setPen(QPen(Qt::black, linWidth));
    rec->setBrush(Qt::white);
    rec->addLabel(0, nx, ny, attribs.value(0), false);
    rec->label(0)->setZ(3);
    rec->addLabel(1, ptx, pty, label2, true);
    rec->label(1)->setZ(3);

    rec->setModelPointer(ct);
    rec->setZ(1);
    rec->show();

    if((p == 0) && (t == 0))
      rec->label(1)->setVisible(false);
    else
      rec->label(1)->setVisible(true);

    transID++;
    return true;
  }

  if(qName == "arc")
  {
    // <arc type="in" weight="1" place="p0" transition="t1" weightx="174" weighty="197">
    int w, x, y;
    CRectangleWithLabels* rect;
    CEllipseWithLabels* ell;
    int c = net->placesCount();
    CPlace* place;
    CTransition* trans;

    for(int i = 0; i < c; i++)
    {
      place = net->place(i);
      if(place->name() == attribs.value(2)) break;
      else place = 0;
    }

    c = net->transitionsCount();
    for(int i = 0; i < c; i++)
    {
      trans = net->transition(i);
      if(trans->name() == attribs.value(3)) break;
      else trans = 0;
    }

    if((place == 0) || (trans == 0)) return true;

    w = attribs.value(1).toInt();
    x = attribs.value(4).toInt();
    y = attribs.value(5).toInt();

    // find the rectangle and ellipse
    QCanvasItemList items = netCanvas->allItems();
    for (QCanvasItemList::iterator a = items.begin(); a != items.end(); a++)
    {
      if((*a)->rtti() == 10004) // rectangle with labels
      {
        rect = static_cast<CRectangleWithLabels* >(*a);
        if(rect->modelPointer() != trans)
          rect = 0;
        else break;
      }
    }

    for (QCanvasItemList::iterator a = items.begin(); a != items.end(); a++)
    {
      if((*a)->rtti() == 10005) // ellipse with labels
      {
        ell = static_cast<CEllipseWithLabels* >(*a);
        if(ell->modelPointer() != place)
          ell = 0;
        else break;
      }
    }
    if((rect == 0) || (ell == 0)) return true;

    CLineWithLabels * line;
    line = new CLineWithLabels (0, netCanvas, ell,  rect);

    if(attribs.value(0) == "in")  // wejciowy dla miejsca
    {
      CArc* arc = new CArc(0, trans, place, w);
      line->setModelPointer(arc);
      net->addArc(arc);
      line->setArrowWidth(ahWidth);
      line->setArrowHeight(ahHeight);
      line->setEndArrow(false);
      line->setStartArrow(true);
    }
    else
    {
      CArc* arc = new CArc(0, place, trans, w);
      line->setModelPointer(arc);
      net->addArc(arc);
      line->setArrowWidth(ahWidth);
      line->setArrowHeight(ahHeight);
      line->setEndArrow(true);
      line->setStartArrow(false);
    }

    line->setPen(QPen(Qt::black,linWidth));
    line->setBrush(Qt::black);
    line->setZ(2);

    //CLabel* label = new CLabel(0, x, y, attribs.value(1), false, netCanvas);
    line->addLabel(0, x, y, attribs.value(1), false);
    if(w > 1)
    {
      (line->label(0))->setVisible(true);
    }
    line->show();
    line->update();
    currLine = line;
    return true;
  }

  if(qName == "breakpoint")
  {
    if(currLine != 0)
    {
      // <breakpoint x="217" y="104"/>
      int x, y, i;
      CSegment* segm;

      x = attribs.value(0).toInt();
      y = attribs.value(1).toInt();

      i = currLine->breaksCount();
      segm = currLine->segment(i);
      currLine->addBreakPoint(segm);
      currLine->moveBreakPoint(i + 2, x, y);

    }
    return true;
  }

  return true;
}


//--------------------------------------------------------------------------
/** The reader calls this function when it has parsed an end element tag
    with the qualified name \a qName, the local name \a localName and the
    namespace URI \a namespaceURI. */

bool CXMLReader::endElement(const QString &/*namespaceURI*/,
  const QString &/*localName*/, const QString &qName)
{
  if(qName == "arc")
  {
    currLine->update();
    currLine = 0;
  }
  if(qName == "net")
  {
    QCanvasItemList items = netCanvas->allItems();
    for (QCanvasItemList::iterator a = items.begin(); a != items.end(); a++)
    {
      (*a)->setSelected(false);
    }

    netView->setID(placeID, transID);
    netCanvas->update();

  }

  return true;
}


//--------------------------------------------------------------------------
/** The reader calls this function when it has parsed a chunk of character data. */

bool CXMLReader::characters(const QString &/*str*/)
{
  return true;
}


//--------------------------------------------------------------------------
/** A reader must use this function to report a non-recoverable error. */

bool CXMLReader::fatalError(const QXmlParseException &/*exception*/)
{
  return true;
}
















