#ifndef _SVG_CIRCLE
#define _SVG_CIRCLE

#include "SVGElement.h"

namespace SVGOutput
{

/** This class represents ellipse element of SVG Document.
 It symbolizes place of a net. */
 
class SVGEllipse : public SVGElement
{
	int rx, ry;
	
	int capacity, marking;
	QString type;

	void validate()
	{
		if(rx < 0)
			rx = 0;
			
		if(ry < 0)
			ry = 0;			
	}
public:
	/** Creates default placed ellipse with style class 
	 set to "ellipse".  */
	SVGEllipse(): SVGElement(0, 0), rx(0), ry(0) 
	{ 
	    setStyleClass("ellipse");
	    capacity = marking = 0;
	}

	/** Creates ellipse with style class set to "ellipse"
	 and assigns it user specified position data. */
	SVGEllipse(int cx, int cy, int rx, int ry) : SVGElement(cx, cy), rx(rx), ry(ry) 
	{ 
	    setStyleClass("ellipse"); 
	    capacity = marking = 0;
	}

	XMLTag *next()
	{
		if(initialized)
		{
			validate();

			XMLTag *tag = SVGElement::next();
			tag->setName("ellipse");

			tag->renameAttribute("x", "cx");
			tag->renameAttribute("y", "cy");
			tag->addAttribute("rx", QString("%1").arg(rx));
			tag->addAttribute("ry", QString("%1").arg(ry));
			
			if(!type.compare("place"))
			{
			    tag->addAttribute("type", "place");
			    tag->addAttribute("capacity", QString("%1").arg(capacity));
			    tag->addAttribute("marking", QString("%1").arg(marking));
			}
			else if(!type.isEmpty())
			    tag->addAttribute("type", type);
			
			return tag;
		}
		else
			return NULL;
	}

	/** Sets the x-axis radius of the ellipse. */
	void setRadiusX(int rx) { this->rx = rx; }

	/** Gets the x-axis radius of the ellipse. */
	int getRadiusX() { return rx; }

	/** Sets the y-axis radius of the ellipse. */
	void setRadiusY(int ry) { this->ry = ry; }
	
	/** Gets the y-axis radius of the ellipse. */
	int getRadiusY() { return ry; }	

	/** Sets capacity of place represented by this ellipse */	
	void setCapacity(int capacity) { this->capacity = capacity; }
	
	/** Gets capacity o of place */
	int getCapacity() { return capacity; }	

	/** Sets initial marking of place represented by this ellipse */
	void setMarking(int marking) { this->marking = marking; }
	
	/** Gets place marking */
	int getMarking() { return marking; }	

	/** 
	 *  Sets type of element represented by this ellipse. 
	 *  Usually you want to use 'dot' or 'place'.
	 */
	void setType(QString value) { type = value; }
	
	/** Returns type of element represented by this ellipse */
	QString getType() { return type; }	
};

} // end of namespace SVGOutput

#endif // _SVG_CIRCLE
