#include <iostream>
#include "SVGExporter.h"

#include <qtextstream.h>

#include <math.h>

namespace SVGOutput
{

void SVGExporter::exportNet(QCanvas *canvas)
{
	// Width and height of document
	int docWidth, docHeight;
	
	if(width < 0)
		docWidth = (int) canvas->width();
	else
	    docWidth = width;
	
	if(height < 0)    
		docHeight = (int) canvas->height();
	else
	    docHeight = height;
	
	// Some setup work
	current = new SVGDocument(docWidth, docHeight, "");
	SVGDocument &doc = *current;
	
	doc.setExternalStyleSheet(externalStyleSheet);
	doc.setExternalScriptUrl(externalScript);
	doc.setExternalScriptType(scriptType);
	doc.setScriptEntryPoint("init()");
	
	if(!viewBox.isEmpty())
	    doc.setViewBox(viewBox);
	else
	    doc.setViewBox(0, 0, (int) (docWidth *scale), (int) (docHeight *scale));	    
	
	// Writer for writing current/doc to file
	XMLTextWriter writer(doc);

	QCanvasItemList items = canvas->allItems();
	QCanvasItemList::iterator it = items.begin();

	while(it != items.end())
	{
	    if((*it)->rtti() == 10005)
		writeEllipseWithLabels((CEllipseWithLabels *) (*it));
	    else if((*it)->rtti() == 10004)
		writeRectangleWithLabels((CRectangleWithLabels *) (*it));
    	    else if (((*it)->rtti() == 10013) && (((CSegment *)(*it))->tag() == 0))
	    {
		CSegment *s;
		if((s = (CSegment*)(*it)))
		{
		    CLineWithLabels *arc;
		    if ((arc = (CLineWithLabels*)(s->owner())))
			writeLineWithLabels(arc);
		}
	    }
		
	    it++;
	}		

	QTextStream textOut(&out);
	writer.write(textOut);
}

/** Creates label */
SVGText *SVGExporter::writeLabel(QString label, int x, int y) 
{ 
	SVGText *text = new SVGText((int) ((x +3)*scale), (int) ((y +14) *scale));
	text->setText(label);
	
	current->addElement(text);
	return text;
}

/** Creates rectangle */
SVGRectangle *SVGExporter::writeRectangle(int x, int y, int width, int height) 
{ 
    SVGRectangle *rect = new SVGRectangle((int) (x *scale), (int) (y *scale), 
	(int) (width *scale), (int) (height *scale));    
	
    current->addElement(rect);
    return rect;
}

/** Creates dot */
void SVGExporter::writeDot(int x, int y) 
{ 
    x = (int) ((x +3.0) *scale);
    y = (int) ((y +3.0) *scale);
    
    int r = (int) (3.0 *scale);
    
    SVGEllipse *ellipse = new SVGEllipse(x, y, r, r);
    ellipse->setStyleClass("dot");
    ellipse->setType("dot");
    
    current->addElement(ellipse);
}

/** Creates Ellipse with labels */
void SVGExporter::writeEllipseWithLabels(CEllipseWithLabels *element) 
{ 
    // Some setup
    int x = (int) (element->x() *scale);
    int y = (int) (element->y() *scale);

    int width = (int) (element->width() *scale /2.0);
    int height = (int) (element->height() *scale /2.0);
    
    CPlace *placePtr = (CPlace *) element->modelPointer();    
    
    // Ellipse
    SVGEllipse *ellipse = new SVGEllipse(x +width, y +height, width, height);
    current->addElement(ellipse);    

    // Label
    CLabel *label = element->label(0);
    writeLabel(label->text(), (int) label->x(), (int) label->y());

    // Net data
    ///////////
    
    ellipse->setType("place");
    ellipse->setCapacity(placePtr->capacity());
    ellipse->setMarking(placePtr->marking());
    ellipse->setId(label->text());
    
    // Fill ellipse with markings
    switch (placePtr->initialMarking())
    {
	case 0:
	    break;
	
	case 1:
	    writeDot((int)(element->x() + (element->width() - 6) / 2), (int)(element->y() + (element->height() - 6) / 2));
	    break;
	
	case 2:
	    writeDot((int)(element->x() + (element->width() - 6) / 3), (int)(element->y() + (element->height() - 6) / 2));
	    writeDot((int)(element->x() + 2 * (element->width() - 6) / 3),	(int)(element->y() + (element->height() - 6) / 2));
	    break;
	
	case 3:
	    writeDot((int)(element->x() + (element->width() - 6) / 2), (int)(element->y() + (element->height() - 6) / 3) - 1);
	    writeDot((int)(element->x() + (element->width() - 6) / 3), (int)(element->y() + 2 * (element->height() - 6) / 3) + 1);
	    writeDot((int)(element->x() + 2 * (element->width() - 6) / 3), (int)(element->y() + 2 * (element->height() - 6) / 3) + 1);
	    break;
	
	case 4:
	    writeDot((int)(element->x() + (element->width() - 6) / 3), (int)(element->y() + (element->height() - 6) / 3) - 1);
	    writeDot((int)(element->x() + 2 * (element->width() - 6) / 3), (int)(element->y() + (element->height() - 6) / 3) - 1);
	    writeDot((int)(element->x() + (element->width() - 6) / 3), (int)(element->y() + 2 * (element->height() - 6) / 3) + 1);
	    writeDot((int)(element->x() + 2 * (element->width() - 6) / 3), (int)(element->y() + 2 * (element->height() - 6) / 3) + 1);
	    break;

	default:
	{
	    QString dotsLabel;
	    dotsLabel = QString::number(placePtr->initialMarking());

	    int x = (int) ((element->x() +element->width() /2 -10) *scale);
	    int y = (int) ((element->y() +element->height() /2 +9 -3) *scale);

	    SVGText *text = new SVGText(x, y);
	    text->setText(dotsLabel);
	    text->setStyleClass("marking");
	    text->setType("marking");
	    text->setParent(label->text());
	
	    current->addElement(text);
	}
    }
}

/** Creates rectangle with labels */
void SVGExporter::writeRectangleWithLabels(CRectangleWithLabels *element) 
{ 
    CTransition *transPtr = (CTransition *) element->modelPointer();    

    SVGRectangle *rect = writeRectangle((int) element->x(), 
	(int) element->y(), element->width(), element->height());

    CLabel *label = element->label(0);
    writeLabel(label->text(), (int) label->x(), (int) label->y());

    // Net data
    ///////////
    	
    rect->setTransition(true);
    rect->setTime(transPtr->time());
    rect->setPriority(transPtr->priority());
    rect->setId(label->text());
    			
    if(element->labelsCount() >1 && element->label(1)->isVisible())
    {
	CLabel *label2 = element->label(1);
	writeLabel(label2->text(), (int) label2->x(), 
		(int) label2->y())->setParent(label->text());
	
	writeRectangle((int) label2->x(), (int) label2->y(), 
	    label2->width() -1, label2->height() -1);
    }

}

/** Creates Line with labels */
void SVGExporter::writeLineWithLabels(CLineWithLabels *arc) 
{ 
    if(CArc* arcPtr = static_cast<CArc* >(arc->modelPointer()))
    {
	// Polyline for rendering arc
	SVGPolyline *line = new SVGPolyline();

	// First point
        int x = (int) (arc->breakpoint(-2).x() *scale);
        int y = (int) (arc->breakpoint(-2).y() *scale);	
	line->insertPoint(x, y);
	
	// Subsequent points
	for (int i=0; i<arc->breaksCount(); i++)
	{
    	    x = (int) (arc->breakpoint(i).x() *scale);
    	    y = (int) (arc->breakpoint(i).y() *scale);	
	    line->insertPoint(x, y);
	}

	// Last point
        x = (int) (arc->breakpoint(-1).x() *scale);
    	y = (int) (arc->breakpoint(-1).y() *scale);
	line->insertPoint(x, y);

	// Net data
	///////////
	
	line->setArc(true);
	line->setWeight(arcPtr->weight());
	line->setPlace(arcPtr->place()->name());
	line->setTransition(arcPtr->transition()->name());
	line->setDirection(
	    (arcPtr->direction() == adPT ? "transition" : "place"));
	
	// Finished with line
	current->addElement(line);

	// Creating arrowhead
	/////////////////////

	// Size of arrowhead
	arrowWidth = 10 *scale;
	arrowHeight = 13 *scale;
	
	// Indices of polygon
	float _x[3], _y[3];

	// Helper vector
	float vecX, vecY;
	
	// Coordinates of last two points of arc
	float startX, startY;
	float endX, endY;

	// Retrieveing coordinates of last points of arc
	if(arcPtr->direction() == adTP)
	{
	    endX = line->getPointX(0);
	    endY = line->getPointY(0);
	    
	    startX = line->getPointX(1);
	    startY = line->getPointY(1);
	}
	else
	{
	    endX = line->getPointX(line->length() -1);
	    endY = line->getPointY(line->length() -1);

    	    startX = line->getPointX(line->length() -2);
	    startY = line->getPointY(line->length() -2);	
	}
	
	// Creates vector pointing along last
	// line of arc to which arrowhead is attached
	vecX = endX;
	vecY = endY;
	
	vecX -= startX;
	vecY -= startY;

	// Normalizing the vector
	float d = sqrtf(vecX*vecX +vecY*vecY);
	vecX = (float) vecX /d;
	vecY = (float) vecY /d; 
	    
	// Position of tip of the arrowhead
	_x[0] = endX;
	_y[0] = endY;
	    
	// Translating base of arrowhead
	// along vector [vecX, vecY]
        _x[1] = _x[0] -vecX *arrowHeight;
	_y[1] = _y[0] -vecY *arrowHeight;
	    
	_x[2] = _x[0] -vecX *arrowHeight;
	_y[2] = _y[0] -vecY *arrowHeight;
	    
	// Rotating [vecX, vecY] by 90 deg.
	float tmp = vecX;
	vecX = vecY;
	vecY = -tmp;
	    
	// Translating points the base of arrowhead consists of
	_x[1] += vecX *(arrowWidth /2);
	_y[1] += vecY *(arrowWidth /2);

	_x[2] -= vecX *(arrowWidth /2);
	_y[2] -= vecY *(arrowWidth /2);

	// Polygon for the arrowhead
	SVGPolygon *poly = new SVGPolygon();
	for(int i=0; i<3; i++)
	    poly->insertPoint((int) _x[i], (int) _y[i]);

	current->addElement(poly);
	
	// Creating elements for additional labels
	if(arc->labelsCount() >0 && arc->label(0)->isVisible())
	    writeLabel(arc->label(0)->text(), (int) arc->label(0)->x(), 
		(int) arc->label(0)->y());
    }
}

} // end of namespace SVGExport
