#ifndef _SVG_EXPORTER
#define _SVG_EXPORTER

#include <qfile.h>
#include <qcanvas.h>

#include "XMLTag.h"
#include "XMLTextWriter.h"
#include "SVGDocument.h"
#include "SVGRectangle.h"
#include "SVGEllipse.h"
#include "SVGText.h"
#include "SVGPolyline.h"
#include "SVGPolygon.h"

#include "../../view/cellipsewithlabels.h"
#include "../../view/crectanglewithlabels.h"
#include "../../view/clinewithlabels.h"
#include "../../model/carc.h"

namespace SVGOutput
{

/** Converts net to svg representation. */

class SVGExporter
{    
private:
    QFile &out;
    SVGDocument *current;

    QString externalStyleSheet;
    QString externalScript, scriptType;

    int width, height;
    QRect viewBox;
            
    float scale;
    float arrowWidth, arrowHeight;

    SVGText *writeLabel(QString, int, int);
    SVGRectangle *writeRectangle(int, int, int, int);
    void writeDot(int, int);

    void writeEllipseWithLabels(CEllipseWithLabels* );
    void writeRectangleWithLabels(CRectangleWithLabels* );
    void writeLineWithLabels(CLineWithLabels* );
    
public:
    /** Creates exporter for writing to @a out file.*/
    SVGExporter(QFile &out): out(out) { scale = 10; current = NULL; }

    ~SVGExporter() { if(current) delete current; }
    
    /** Writes net representation from QCanvas object to file. */
    void exportNet(QCanvas *);
    
    /** Sets scale. */
    void setScale(float scale) { this->scale = scale; }

    /** Gets scale. */
    float getScale() { return scale; }

    
    /** Sets link to external style sheet file. */    
    void setExternalStyleSheet(QString css) { externalStyleSheet = css; }

    /** Gets link to external style sheet file. */
    QString getExternalStyleSheet() { return externalStyleSheet; }
    
    /** Sets link to external script file. */
    void setExternalScriptUrl(QString url) { externalScript = url; }

    /** Gets link to external script file. */
    QString getExternalScriptUrl() { return externalScript; }

    /** Sets external script type. */
    void setExternalScriptType(QString type) { scriptType = type; }

    /** Gets external script type. */
    QString getExternalScriptType() { return scriptType; }
    
    /** Sets width of created document */
    void setWidth(int width) { this->width = width; }
    
    /** Gets width of created document */
    int getWidth() { return width; }
    
    /** Sets height of created document */
    void setHeight(int height) { this->height = height; }
    
    /** Gets height of created document */
    int getHeight() { return height; }
    
    /** Sets viewBox of generated document */
    void setViewBox(QRect rect) { viewBox = rect; }
    
    /** Gets viewBox of generated document */
    QRect getViewBox() { return viewBox; }
};

} // end of namespace SVGExport

#endif // _SVG_EXPORTER
