#ifndef _SVG_TEXT
#define _SVG_TEXT

#include <qvaluevector.h>
#include <qstring.h>

#include "SVGElement.h"

namespace SVGOutput
{

/**
 Represents 'text' element of SVG Document.
 The 'text' element defines a graphics element consisting of text.
 */
class SVGText : public SVGElement
{
	QString text, font;
	QString parent, type;

	QValueVector<XMLTag *> tags;
	QValueVector<XMLTag *>::Iterator it;

public:
	/** Creates empty text element. */
	SVGText() { setStyleClass("text"); }

	/** Creates empty text element at position @a x and @a y. */
	SVGText(int x, int y): SVGElement(x, y) { setStyleClass("text"); }

	~SVGText() 
	{ 
	    it = tags.begin();	    
	    while(it != tags.end())
	    {
			delete *it;
			it++;
	    }
	}
	
	/** Sets text of an element. */
	void setText(QString text) { this->text = text; }

	/** Gets text of an element. */
	QString getText() { return text; }

	/** Sets font familly to use when dislaying this element. */
	void setFontFamily(QString font) { this->font = font; }

	void init() 
	{
		SVGElement::init();
		XMLTag *tag = SVGElement::next();
		
		tag->setName("text");
		tag->setType(XMLTag::ttOpening);

		if(!font.isEmpty())
		    tag->addAttribute("font-family", font);

		if(!parent.isEmpty())
		    tag->addAttribute("parent", parent);

		if(!type.isEmpty())
		    tag->addAttribute("type", type);
		    
		tags.push_back(tag);

		XMLTag *txt = new XMLTag(text, XMLTag::ttText);
		tags.push_back(txt);

		tags.push_back(new XMLTag("text", XMLTag::ttClosing));

		it = tags.begin();
	}

	void reset() { it = tags.begin(); }

	XMLTag *next() { return *it++; }
	bool hasNext() { return it != tags.end(); }

	/** Sets id of parent shape */
	void setParent(QString parent) { this->parent = parent; }
	
	/** Gets id of parent shape. */
	QString getParent() { return parent; }

	/** Sets type of element represented by this text, e.g. 'marking'. */
	void setType(QString type) { this->type = type; }

	/** Gets type of element represented by this text, e.g. 'marking'. */
	QString getType() { return type; }		
};

} // end of namespace SVGOutput

#endif // _SVG_TEXT
