#ifndef _XML_TAG
#define _XML_TAG

#include <qstring.h>
#include <qpair.h>
#include <qvaluevector.h>

namespace SVGOutput
{

/** 
 Defines vector of tag's attributes. Vector is preffered
 over Map because some svg elements 
 */
typedef QValueVector<QPair<QString, QString> > attributes_map;
typedef QPair<QString, QString> string_pair;

/**
 Defines XML tags of various types.
 */
class XMLTag
{
public:
	/** Defines tag types */
	enum TagType { 
		ttOpening, /*!< Opening tag. */
		ttClosing, /*!< Closing tag. */
		ttText, /*!< Just text. */
		ttComment, /*!< XML Comment. */
		ttElement, /*!< Self containing element. */
		ttInstruction, /*!< Processing instruction. */
		ttDocumentType  /*!< Document type definition. */
	};

	/** Constructs new tag, named @a name of @a type, with optional @a id. */
	XMLTag(QString name, TagType type, QString id ="")
		:name(name), type(type) { id = id; }

	/** Gets name. */
	QString getName() { return name; }
	/** Sets name. */
	void setName(QString name) { this->name = name; }

	/** Gets tag type. */
	TagType getType() { return type; }

	/** Sets tag type. */
	void setType(TagType type) { this->type = type; }

	/** Adds new attribute. */
	void addAttribute(QString key, QString value)
	{
		attributes_map::iterator it = attributes.begin();
		while(it != attributes.end())
		{
			if(!it->first.compare(key))
			{
				it->second = value;
				return;
			}
			it++;
		}

		attributes.push_back(string_pair(key, value));
	}

	/** Retrieves value coresponding to @a key. */
	QString getAttribute(QString key)
	{
		attributes_map::iterator it = attributes.begin();
		while(it != attributes.end())
		{
			if(!it->first.compare(key))
				return it->second;
			it++;
		}

		return QString("");
	}

	/** Renames attribute. */
	void renameAttribute(QString oldKey, QString newKey)
	{
		attributes_map::iterator it = attributes.begin();
		while(it != attributes.end())
		{
			if(!it->first.compare(oldKey))
			{
				it->first = newKey;
				return;
			}
			it++;
		}
	}

	/** Removes attribute. */
	void removeAttribute(QString key)
	{
		attributes_map::iterator it = attributes.begin();
		while(it != attributes.end())
		{
			if(!it->first.compare(key))
			{
				attributes.erase(it);
				return;
			}
			it++;
		}
	}
	
	/** Returns attribute map of this element */
	attributes_map &getAttributesMap() { return attributes; }

private:
	QString name;
	QString id;

	TagType type;

	attributes_map attributes;
};

} // end of namespace SVGOutput

#endif // _XML_TAG
