/***************************************************************************
    copyright            : (C) 2004 - 2005 by Marcin Szpyrka
    email                : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CELLIPSE_H
#define CELLIPSE_H

#include <qcanvas.h>
#include <qpainter.h>
#include <qcolor.h>
#include <qpointarray.h>
#include "celement.h"

/**
  The CEllipse class provides an ellipse on a QCanvas.
  The class inherits methods from QCanvasPolygonalItem.
*/
class CEllipse : public QCanvasPolygonalItem, public CElement
{
protected:
  int    ellipseWidth;
  int    ellipseHeight;
  int    dots;
  QFont  dotsLabelFont;

public:
  enum { RTTI = 10002 };

  /** Constructs an ellipse. */
  CEllipse(int etag, QCanvas* canvas) :
    QCanvasPolygonalItem(canvas), CElement(etag)
  {
    dotsLabelFont = QFont("times");
  }

  CEllipse(int etag, int x, int y, int w, int h, QCanvas* canvas);

  /** Destroys the ellipse. */
  ~CEllipse(){ hide(); }

  /** Returns CEllipse class RTTI */
  virtual int rtti() const { return RTTI; }

  void drawShape(QPainter & painter);
  QPointArray areaPoints() const;

  /** Returns the width of the ellipse. */
  int width() const { return ellipseWidth; }

  /** Returns the height of the ellipse. */
  int height() const { return ellipseHeight; }

  /** Sets the width of the ellipse. If \a w < 0, the ellipse width = 0 is set. */
  void setWidth(int w){ ellipseWidth = w > 0 ? w : 0; update(); }

  /** Sets the height of the ellipse. If \a h < 0, the ellipse height = 0 is set. */
  void setHeight(int h){ ellipseHeight = h > 0 ? h : 0;  update(); }
  
  void setDots(int d);
  void setColor(const QColor & c);

  /** Sets the font for dots label. */
  void setFont(const QFont & f){ dotsLabelFont = f; }
};

#endif





