/***************************************************************************
    copyright            : (C) 2004 - 2005 by Marcin Szpyrka
    email                : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CELLIPSEWITHLABELS_H
#define CELLIPSEWITHLABELS_H

#include "cellipse.h"
#include "clabel.h"
#include <qptrlist.h>
#include "../model/cplace.h"

/**
  The CEllipseWithLabels class provides an ellipse on a QCanvas
  with a list of labels connected with it.
*/

class CEllipseWithLabels : public CEllipse
{
protected:
  QPtrList<CLabel> labels;
public:
  enum { RTTI = 10005 };

  /** Constructs an ellipse. */
  CEllipseWithLabels(int etag, QCanvas* canvas) :
      CEllipse(etag,canvas){}

  /**
  Constructs an ellipse at position (\a x, \a y) and
  \a w by \a h, on \a canvas
  \param etag the ellipse tag,
  \param x the horizontal position of the ellipse,
  \param y the horizontal position of the ellipse,
  \param w the ellipse width,
  \param h the ellipse height,
  \param canvas the parent canvas.
  */
  CEllipseWithLabels(int etag, int x, int y, int w, int h, QCanvas* canvas):
      CEllipse(etag, x, y, w, h, canvas){}

  /** Destroys the ellipse. */
  ~CEllipseWithLabels(){ }

  /** Returns CEllipseWithLabels class RTTI */
  int rtti() const { return RTTI; }

  // adding labels
  bool addLabel(CLabel* lab);
  bool addLabel(int ltag, int x, int y, QString text, bool yes);

  // removing labels
  bool removeLabel(int ltag);
  bool removeLabel(CLabel* lab);
  void removeAllLabels();

  // access to labels
  /**
  Returns a pointer to the label at position \a index in the list,
  or 0 if the index is out of range. The valid range is 0..(labelsCount() - 1)
  inclusive.
  */
  CLabel* label(int index) { return labels.at(index); }

  /** Returns the number of labels. */
  int labelsCount() const { return labels.count(); };

  // moving elements
  void move(double nx, double ny);
  void moveBy(double dx, double dy);
  void show();
};

#endif












