/***************************************************************************
    copyright            : (C) 2004 - 2005 by Marcin Szpyrka
    email                : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "chandle.h"

//--------------------------------------------------------------------------
/** Constructs a handle at position (\a x, \a y) and
    size 2* \a d by 2 * \a d that belongs to owner \a o. */
    
CHandle::CHandle(QCanvas *canvas, void* o, int x, int y, int d) :
      QCanvasRectangle(int(x), int(y), int(2*d), int(2*d), canvas)
{
  xc = int(this->x()) + d;
  yc = int(this->y()) + d;
  rad = d;
  own = o;
  htag = -1;
}


//--------------------------------------------------------------------------
/** Sets the color of the handle. */

void CHandle::setColor(const QColor & c)
{ 
  QPen p = pen();
  p.setWidth(pen().width());
  p.setColor(c);
  setPen(p);
}


//--------------------------------------------------------------------------
/** Sets the center point of the handle. */

void CHandle::setCenter(int nx, int ny)
{
  this->move(nx - rad, ny - rad);
}


//--------------------------------------------------------------------------
/** Moves the handle relative to its current position by (dx, dy). */

void CHandle::moveBy(double dx, double dy)
{
  QCanvasRectangle::moveBy(dx, dy);
  xc = int(x()) + rad;
  yc = int(y()) + rad;
}


//--------------------------------------------------------------------------
/** Moves the handle relative to its current position by (dx, dy). */

void CHandle::move(double nx, double ny)
{
  QCanvasRectangle::move(nx, ny);
  xc = int(x()) + rad;
  yc = int(y()) + rad;
}




