/***************************************************************************
    copyright            : (C) 2004 - 2005 by Marcin Szpyrka
    email                : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CHANDLE_H
#define CHANDLE_H

#include <qcanvas.h>
#include <qpainter.h>
#include <qcolor.h>


/**
  The CHandle class provides a handle for a line.
  The class inherits methods from QCanvasRectangle.
*/
class CHandle : public QCanvasRectangle
{
protected:
  int    htag;
  int    xc;
  int    yc;  
  int    rad;
  void*  own;
  
public:

  enum { RTTI = 10011 };

  /** Constructs an empty handle. */
  CHandle(QCanvas *canvas) : QCanvasRectangle(canvas), own(0){}

  CHandle(QCanvas *canvas, void* o, int x, int y, int d);

  /** Destroys the CRectangle. */
  ~CHandle(){ hide(); }
  
  /** Returns the tag of the element. */
  int tag() const { return htag; }
  
  /** Sets the element tag to \a etag. */
  void setTag(int tag){ htag = tag; }

  /** Returns CHandle class RTTI */
  virtual int rtti() const { return RTTI; }

  void setColor(const QColor & c);

  void setCenter(int nx, int ny);
  
  /** Returns the \a x entry of the center point. */
  int xCenter(){ return int(xc); }
  
  /** Returns the \a y entry of the center point. */
  int yCenter(){ return int(yc); }
  
  /** Returns a pointer to the handle owner. */
  void* owner(){ return own; }
  
  void moveBy(double dx, double dy);
  void move(double nx, double ny);
};

#endif

