/***************************************************************************
    copyright            : (C) 2004 - 2005 by Marcin Szpyrka
    email                : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CLABEL_H
#define CLABEL_H

#include <qcanvas.h>
#include <qpainter.h>
#include <qcolor.h>
#include <qstring.h>
#include <qfontmetrics.h>


/**
  The CLabel class provides a label (with or without a frame)
  on a QCanvas. The class inherits methods from QCanvasRectangle.
*/

class CLabel : public QCanvasRectangle
{
protected:
  int     labelTag;
  QString labelText;
  bool    labelFrame;
  QFont   labelFont;
public:
  enum { RTTI = 10003 };

  /**
    Constructs an empty label at position (0,0) with both
    width and height set to 0 pixels on \a canvas.
  */
  CLabel(int ltag, QCanvas* canvas) : QCanvasRectangle(0,0,0,0,canvas)
  {
    labelTag = ltag;
    labelText = "";
    labelFrame = false;
    labelFont = QFont("times");
  }

  CLabel(int ltag, int x, int y, QString text, bool yes, QCanvas* canvas);

  /** Destroys the label. */
  ~CLabel(){ hide(); }

  /** Returns CLabel class RTTI */
  int rtti() const { return RTTI; }

  void draw(QPainter & painter);
  QPointArray areaPoints() const;

  /** Returns true if the label is placed inside a box,
    or false otherwise. */
  bool frame() const { return labelFrame; }

  /** Returns the text of the label. */
  QString text() const { return labelText; }

  /** Returns the tag of the label. */
  int tag() const { return labelTag; }

  void setTag(int ltag);
  void setFrame(bool yes);
  void setText(QString t);
  void setColor(const QColor & c);
  void setFont(const QFont & f);
  void update(){QCanvasRectangle::update();}
};

#endif
