/***************************************************************************
    copyright            : (C) 2004 - 2005 by Marcin Szpyrka
    email                : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cline.h"
#include <math.h>


//--------------------------------------------------------------------------
/** Creates a line that connects an ellipste pointed by \a ellipPtr 
  with a rectangle pointed by \a recPtr. */

CLine::CLine(int ltag, QCanvas* canvas, QCanvasPolygonalItem *firstPtr, 
             QCanvasPolygonalItem *secondPtr) : CElement(ltag), points(2)
{
  lineCanvas = canvas; 
  lineSelected = false;
  lineVisible = false;
  lineZ = 0;
  breaks = 0;
  firstNode  = firstPtr;
  secondNode = secondPtr;
    
  CSegment *segment;
  segment = new CSegment(0, canvas, this);
  segments.append(segment);
  
  CHandle *hand;
  hand = new CHandle(canvas,this, 0, 0, 3);
  hand->setTag(0);
  hand->setBrush(QBrush(QColor(0, 0, 200), Qt::SolidPattern));
  hand->setPen(QPen (Qt::NoPen));
  handles.append(hand);
  
  hand = new CHandle(canvas,this, 0, 0, 3);
  hand->setTag(0);
  hand->setBrush(QBrush(QColor(0, 0, 200), Qt::SolidPattern));
  hand->setPen(QPen (Qt::NoPen));
  handles.append(hand);
    
  QCanvasPolygon *arrow;
  arrow = new QCanvasPolygon(canvas); // start arrowhead
  arrow->setVisible(false);
  arrowheads.append(arrow);
  arrow = new QCanvasPolygon(canvas); // end arrowhead
  arrow->setVisible(false);
  arrowheads.append(arrow);
  
  points.setPoint(0,0,0);
  points.setPoint(1,0,0);
  
  findPoints();
}
  
  
//--------------------------------------------------------------------------
/** Destroys the line. */  
 
CLine::~CLine()
{
  CHandle* h;
  CSegment* s;
  QCanvasPolygon* a;

  for(a = arrowheads.first(); a; a = arrowheads.next())
  {
    delete a;
  }
  for(h = handles.first(); h; h = handles.next())
  {
    delete h;
  }
  for(s = segments.first(); s; s = segments.next())
  {
    delete s;
  }

  arrowheads.clear();
  handles.clear();
  segments.clear();
}  
  
  
//--------------------------------------------------------------------------
/** Computes the characteristic points of the line. */

void CLine::findPoints()
{
  double xc1, yc1, xc2, yc2;
  double x1, y1;
  QPoint p;
  CEllipseWithLabels*   ellipse1;
  CRectangleWithLabels* rectangle1;
  CEllipseWithLabels*   ellipse2;
  CRectangleWithLabels* rectangle2;
  
  points.resize(breaks + 2);

  // break points
  if(breaks > 0)
  {
    CHandle *hand;
    for(int i = 0; i < breaks; i++)
    {
      hand = handles.at(i + 2);
      points.setPoint(i + 2, hand->xCenter(), hand->yCenter());
    }
  }
  
  if(secondNode->rtti() == 10005) // ellipse with labels
  {
    ellipse2 = static_cast<CEllipseWithLabels*>(secondNode);
    xc2 = ellipse2->x() + ellipse2->width() / 2;// centers of the nodes
    yc2 = ellipse2->y() + ellipse2->height() / 2;
  }
  else if(secondNode->rtti() == 10004) // rectangle with labels
  {
    rectangle2 = static_cast<CRectangleWithLabels*>(secondNode);
    xc2 = rectangle2->x() + rectangle2->width() / 2;// centers of the nodes
    yc2 = rectangle2->y() + rectangle2->height() / 2;
  }
  
  if(firstNode->rtti() == 10005) // ellipse with labels
  {
    ellipse1 = static_cast<CEllipseWithLabels*>(firstNode);
    xc1 = ellipse1->x() + ellipse1->width() / 2;
    yc1 = ellipse1->y() + ellipse1->height() / 2;
  
    // the point on the ellipse
    if(breaks > 0)
    {
      p = points.point(2);
      x1 = p.x();
      y1 = p.y();
    }
    else
    {
      x1 = xc2;
      y1 = yc2;
    }
    
    p = ellipsePoint(ellipse1->width()/2, ellipse1->height()/2,
      xc1, yc1, x1, y1);
      
    points.setPoint(0, p.x(), p.y());
  }
  else if(firstNode->rtti() == 10004) // rectangle with labels
  {
    rectangle1 = static_cast<CRectangleWithLabels*>(firstNode);
    xc1 = rectangle1->x() + rectangle1->width() / 2;
    yc1 = rectangle1->y() + rectangle1->height() / 2;
    
    // the point on the rectangle
    if(breaks > 0)
    {
      p = points.point(2);
      x1 = p.x();
      y1 = p.y();
    }
    else
    {
      x1 = xc2;
      y1 = yc2;
    }
    
    p = rectanglePoint(rectangle1->width()/2, rectangle1->height()/2,
      xc1, yc1, x1, y1);
      
    points.setPoint(0, p.x(), p.y());
  }

  // second node
  if(secondNode->rtti() == 10005) // ellipse with labels
  {
    // the point on the ellipse
    if(breaks > 0)
    {
      p = points.point(2);
      x1 = p.x();
      y1 = p.y();
    }
    else
    {
      x1 = xc2;
      y1 = yc2;
    }
    
    p = ellipsePoint(ellipse2->width()/2, ellipse2->height()/2,
      xc1, yc1, x1, y1);
      
    points.setPoint(1, p.x(), p.y());
  }
  else if(secondNode->rtti() == 10004)
  {
    // the point on the rectangle
    if(breaks > 0)
    {
      QPoint p = points.point(breaks + 1);
      x1 = p.x();
      y1 = p.y();
    }
    else
    {
      x1 = xc1;
      y1 = yc1;
    }
    
    p = rectanglePoint(rectangle2->width()/2, rectangle2->height()/2,
      xc2, yc2, x1, y1);
      
    points.setPoint(1, p.x(), p.y());
  }  
}


//--------------------------------------------------------------------------
/** Sets the arrow width to \a width. */

void CLine::setArrowWidth(int width)
{
  aWidth = width;
}


//--------------------------------------------------------------------------
/**   Sets the arrow height to \a height. */

void CLine::setArrowHeight(int height)
{
  aHeight = height;
}


//--------------------------------------------------------------------------
/** Sets the selected flag of the item to \a yes. */

void CLine::setSelected(bool yes)
{
  lineSelected = yes;
  CHandle *i;
  
  for(i = handles.first(); i; i = handles.next())
  {
    i->setVisible(yes);
  } 
}


//--------------------------------------------------------------------------
/** Adds a break point to the line. */

void CLine::addBreakPoint(CSegment *s)
{
  QPoint p1, p2;
  p1 = s->startPoint();
  p2 = s->endPoint();
  
  int xn, yn;
  // center of the segment
  xn = (p1.x() + p2.x()) / 2;
  yn = (p1.y() + p2.y()) / 2;
  
  CSegment *segment;
  segment = new CSegment(1, lineCanvas, this);
  segment->setVisible(true);
  segment->setPen(linePen);
  segment->setBrush(lineBrush);
  
  CHandle *hand;
  hand = new CHandle(lineCanvas,this, xn - 3, yn - 3, 3);
  hand->setTag(1);
  hand->setBrush(QBrush(QColor(0, 0, 200), Qt::SolidPattern));
  hand->setPen(QPen (Qt::NoPen));
  hand->setZ(lineZ + 1);
  hand->setVisible(lineVisible);
  
  int i = points.find(p1);   // point index
  if(i == 0)
  {
    handles.insert(2, hand);
    segments.insert(1, segment);
  }
  else
  {
    handles.insert(i + 1, hand);
    segments.insert(i, segment);
  }
  breaks++;
    
  update();
}


//--------------------------------------------------------------------------
/** Deletes a break point from the line. */

void CLine::deleteBreakPoint(CHandle *h)
{
  if(h == 0) return;
  
  int index = handles.find(h);
  handles.remove(h);
  delete h;
  
  index = index - 2;
  CSegment* s = segments.at(index);
  segments.remove(s);
  delete s;
  
  breaks--;    
  update();
}


//--------------------------------------------------------------------------
/** Adds the start arrowhead to the line. */
  
void CLine::setStartArrow(bool yes)
{ 
  arrowheads.at(0)->setVisible(yes);
}


//--------------------------------------------------------------------------
/** Adds the end arrowhead to the line. */
  
void CLine::setEndArrow(bool yes)
{ 
  arrowheads.at(1)->setVisible(yes);
}


//--------------------------------------------------------------------------
/** Updates handles and arrows' heads. */
  
void CLine::update()
{
  findPoints(); 
  
  QPointArray aPoints(3);
    
  QPoint p = points.point(0); // first node
  int x1, y1, x2, y2;
  int u1, u2;
  double t, d;
  
  x1 = p.x();
  y1 = p.y();
  
  // the first handle is placed next to the first node  
  CHandle *hand;
  hand = handles.first();
  hand->setCenter(x1, y1);
  
  // find points for the first arrowhead
  aPoints.setPoint(0, x1, y1); // top point
  
  if(breaks == 0)
  {
    p = points.point(1); // the second node
  }
  else
  {
    p = points.point(2); // first break point next to the first node
  }
    
  x2 = p.x();
  y2 = p.y();
  u1 = x2 - x1;    // direction vector of the line that goes through points 1 and 2
  u2 = y2 - y1;
  d = sqrt(u1*u1 + u2*u2);
  if(d < 1.0) d = 1.0;
    
  t = aHeight / d; 
  x2 = x1 + t * u1; // center of the arrowhead base
  y2 = y1 + t * u2;

  t = aWidth / (2 * d);
  x1 = x2 + t * u2; // first point of the arrowhead base
  y1 = y2 - t * u1;
  aPoints.setPoint(1, x1, y1);
  
  x1 = x2 - t * u2; // second point of the arrowhead base
  y1 = y2 + t * u1;
  aPoints.setPoint(2, x1, y1);
  
  arrowheads.at(0)->setPoints(aPoints);

  // the second handle is placed next to the second node 
  p = points.point(1); // the second node
  x1 = p.x();
  y1 = p.y();
  hand = handles.at(1);
  hand->setCenter(x1, y1);
  
  // find points for the second arrowhead
  aPoints.setPoint(0, x1, y1); // top point of the arrowhead
  
  if(breaks == 0)
  {
    p = points.point(0); // the second node
  }
  else
  {
    p = points.point(breaks + 1); // first break point next to the second node 
                                  // (the last break point)
  }
  
  x2 = p.x();
  y2 = p.y();
  u1 = x2 - x1;    // direction vector of the line that goes through points 1 and 2
  u2 = y2 - y1;
  d = sqrt(u1*u1 + u2*u2);
  if(d < 1.0) d = 1;
    
  t = aHeight / d;
  x2 = x1 + t * u1; // center of the arrowhead base
  y2 = y1 + t * u2;

  t = aWidth / (2 * d);
  x1 = x2 + t * u2; // first point of the arrowhead base
  y1 = y2 - t * u1;
  aPoints.setPoint(1, x1, y1);
  
  x1 = x2 - t * u2; // second point of the arrowhead base
  y1 = y2 + t * u1;
  aPoints.setPoint(2, x1, y1);
  
  arrowheads.at(1)->setPoints(aPoints);

  // other break points
  if(breaks > 0)
  {
    for(int i = 0; i < breaks; i++)
    {
      hand = handles.at(i + 2);
      p = points.point(i + 2);
      hand->setCenter(p.x(), p.y());
    }
  }
  
  QPoint p1, p2;
  CSegment* s;
  int j = segments.count();
  for(int i = 0; i < j; i++)
  {
    s = segments.at(i);
    if(j == 1) // there is only one segment
    {
      p1 = points.point(0);
      p2 = points.point(1);
      s->setPoints(p1.x(), p1.y(), p2.x(), p2.y());   
    } 
    else
    {     
      if(i == 0)
      {
        p1 = points.point(0);
        p2 = points.point(2);
        s->setPoints(p1.x(), p1.y(), p2.x(), p2.y());
      }
      else if(i == j-1)
      {
        p1 = points.point(i + 1);
        p2 = points.point(1);
        s->setPoints(p1.x(), p1.y(), p2.x(), p2.y());
      }
      else
      {
        p1 = points.point(i + 1);
        p2 = points.point(i + 2);
        s->setPoints(p1.x(), p1.y(), p2.x(), p2.y());
      }
    }
  }
}


//--------------------------------------------------------------------------
/** Sets the QPen used when drawing the item to the pen \a p. */

void CLine::setPen(QPen p)
{
  linePen = p;
  CSegment* i;
  
  for(i = segments.first(); i; i = segments.next())
  {
    i->setPen(p);
  }
  arrowheads.at(0)->setPen(p);
  arrowheads.at(1)->setPen(p);
}


//--------------------------------------------------------------------------
/** Sets the QBrush used when drawing the polygonal item to the brush \a b. */

void CLine::setBrush(QBrush b)
{
  lineBrush = b;
  CSegment* i;
  
  for(i = segments.first(); i; i = segments.next())
  {
    i->setBrush(b);
  }
  arrowheads.at(0)->setBrush(b);
  arrowheads.at(1)->setBrush(b);
}


//--------------------------------------------------------------------------
/** Sets the QCanvas upon which the canvas item is to be drawn to \a c.*/

void CLine::setCanvas(QCanvas * c)
{
  lineCanvas = c;
  CSegment* i;
  CHandle*  h;
  
  for(i = segments.first(); i; i = segments.next())
  {
    i->setCanvas(c);  
  }
  
  arrowheads.at(0)->setCanvas(c);
  arrowheads.at(1)->setCanvas(c);
  
  for(h = handles.first(); h; h = handles.next())
  {
    h->setCanvas(c);
  }
}


//--------------------------------------------------------------------------
/** Sets the z index of the canvas item to \a z; sets the handles z index
    to \a z+1. */

void CLine::setZ(double z)
{
  lineZ = z;
  CSegment* i;
  CHandle*  h;
  
  for(i = segments.first(); i; i = segments.next())
  {
    i->setZ(z);
  }
  
  arrowheads.at(0)->setZ(z);
  arrowheads.at(1)->setZ(z);
  
  for(h = handles.first(); h; h = handles.next())
  {
    h->setZ(z + 1);
  }
}


//--------------------------------------------------------------------------
/** Makes the canvas item visible if \a yes is \a true, or invisible 
  if \a yes is \a false. The change takes effect when 
  QCanvas::update() is next called. */

void CLine::setVisible(bool yes) 
{ 
  lineVisible = yes; 
  CSegment* i;
  
  for(i = segments.first(); i; i = segments.next())
  {
    i->setVisible(yes);
  }
}


//--------------------------------------------------------------------------
/** Moves the \a i-th breakpoint to (\a x, \a y). */

void CLine::moveBreakPoint(int i, int x, int y)
{
  if((i > 1) && (i < breaks + 2))
  {
    handles.at(i)->setCenter(x, y);
  }
}


//--------------------------------------------------------------------------
/** Returns the i-the breakpoint. */

QPoint CLine::breakpoint(int i)
{ 
  return points.point(i+2); 
} 


//--------------------------------------------------------------------------
/** Returns a point that belongs to the ellipse with the center (\a xc, \a yc )
  and radiuses \a a and \a b, and to line that goes through the points
  (\a xc, \a yc ) and (\a x, \a y ). */

QPoint CLine::ellipsePoint(double a, double b, double xc, double yc, 
  double x, double y)
{  
  double u1, u2, v1, v2, xn, yn;
  double cosPhi, sinPhi;
  QPoint p;
  
  u1 = x - xc;    // fisrt vector
  u2 = y - yc;
  v1 = a;         // second vector
  v2 = 0;
  
  cosPhi = (u1 * v1)/(sqrt(u1*u1 + u2*u2) * v1);
  sinPhi = sqrt(1 - cosPhi * cosPhi); 
   
  /* Check whether the point is over or under the center of the ellipse.
    If the point is over the center, we change the sign of sinPhi. */ 
  if(y < yc) sinPhi = -sinPhi;
  
  xn = xc + a * cosPhi;
  yn = yc + b * sinPhi;
  p.setX(int(xn));
  p.setY(int(yn));
  return p;
}


//--------------------------------------------------------------------------
/** Returns a point that belongs to the rectangle with the center (\a xc, \a yc )
  and sides lengths \a a*2 and \a b*2, and to line that goes through the points
  (\a xc, \a yc ) and (\a x, \a y ). */

QPoint CLine::rectanglePoint(double a, double b, double xc, double yc, 
  double x, double y)
{
  double u1, u2, v1, v2, xn, yn;
  double cosPhi, sinPhi, cosPsi, w1, w2;
  QPoint p;
  
  u1 = x - xc;    // fisrt vector
  u2 = y - yc;
  v1 = a;         // second vector
  v2 = 0;
  
  cosPhi = (u1 * v1)/(sqrt(u1*u1 + u2*u2) * fabs(v1));  
  
  /* Check whether the point is over or under the center of the rectangle.
    If the point is under the center, we compare the angle Phi with
    the angle between vectors:
    1) with start point in the center of the rectangle and the end point
       in the right bottom corner,
    2) with start point in the center of the rectangle and the end point
       in the left bottom corner. */
  if(yc < y)
  {
    // right bottom corner
    w1 = v1;
    w2 = b;
    cosPsi = (w1 * v1)/(sqrt(w1*w1 + w2*w2) * fabs(v1));
    
    if(cosPhi >= cosPsi) // point on the right side
    {
      xn = xc + a;
      yn = yc + ((xn - xc)/u1) * u2;
    }
    else
    {
      // left bottom corner
      w1 = -v1;
      w2 = b;
      cosPsi = (w1 * v1)/(sqrt(w1*w1 + w2*w2) * fabs(v1));
      
      if(cosPhi <= cosPsi) // point on the left side
      {
        xn = xc - a;
        yn = yc + ((xn - xc)/u1) * u2;
      }
      else // point on the bottom side
      {
        yn = yc + b;
        xn = xc + ((yn - yc)/u2) * u1;
      }
    }   
  }
  /* If the point is over the center, we compare the angle Phi with
    the angle between vectors:
    1) with start point in the center of the rectangle and the end point
       in the right top corner,
    2) with start point in the center of the rectangle and the end point
       in the left top corner. */
  else
  {
    // left top corner
    w1 = -v1;
    w2 = -b;
    cosPsi = (w1 * v1)/(sqrt(w1*w1 + w2*w2) * fabs(v1));
    
    if(cosPsi >= cosPhi) // point on the left side
    {
      xn = xc - a;
      yn = yc + ((xn - xc)/u1) * u2;
    }
    else
    {
      // right top corner
      w1 = v1;
      w2 = -b;
      cosPsi = (w1 * v1)/(sqrt(w1*w1 + w2*w2) * fabs(v1));
      
      if(cosPsi <= cosPhi) // point on the right side
      {
        xn = xc + a;
        yn = yc + ((xn - xc)/u1) * u2;
      }
      else // point on the top side
      {
        yn = yc - b;
        xn = xc + ((yn - yc)/u2) * u1;
      }
    }
  }
  p.setX(int(xn));
  p.setY(int(yn));
  return p;
}



