/***************************************************************************
    copyright            : (C) 2004 - 2005 by Marcin Szpyrka
    email                : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CLINE_H
#define CLINE_H

#include <qcanvas.h>
#include <qpainter.h>
#include <qcolor.h>
#include <qpointarray.h>
#include <qpoint.h>
#include <qptrlist.h>
#include <qpen.h> 

#include "celement.h"
#include "crectanglewithlabels.h"
#include "cellipsewithlabels.h"
#include "../model/carc.h"
#include "chandle.h"
#include "csegment.h"


/** The CLine class provides a polyline with arrowheads on a QCanvas.*/

class CLine: public CElement
{
protected:
	QCanvas* lineCanvas; 
	QPen     linePen;
	QBrush   lineBrush;
	bool     lineSelected;
	bool     lineVisible;
	double   lineZ;
	int      breaks;
  int      aWidth;
	int      aHeight;
	
  QCanvasPolygonalItem    *firstNode;
	QCanvasPolygonalItem    *secondNode; 
  QPtrList<QCanvasPolygon> arrowheads;
	QPtrList<CHandle>        handles;
	QPtrList<CSegment>       segments;
  QPointArray              points;

  QPoint ellipsePoint(double a, double b, double xc,
                      double yc, double x, double y);
  QPoint rectanglePoint(double a, double b, double xc,
                        double yc, double x, double y);                    
	void findPoints();
	
public:
	enum { RTTI = 10009 };

	CLine(int ltag, QCanvas* canvas, QCanvasPolygonalItem *firstPtr,	 //???
				QCanvasPolygonalItem *secondPtr);
        
	virtual ~CLine();
	
	/** Returns the QBrush used to fill the item, if filled. */
	QBrush brush() const { return lineBrush; }
	
	void setBrush(QBrush b);
	
	/** Returns the QPen used to draw the outline of the item, if any. */
	QPen pen() const { return linePen; }
	
	void setPen(QPen p);
	
	/** Returns CLine class RTTI */
	virtual int rtti() const { return RTTI; }	
	
	/** Returns the canvas containing the canvas item. */
	QCanvas * canvas() const { return lineCanvas; }
	
	void setCanvas(QCanvas * c);
	
	/** Returns \a true if the canvas item is selected; otherwise returns \a false.*/
	bool isSelected () const { return lineSelected; }
	
	void setSelected(bool yes);
	
	/** Returns \a true if the canvas item is visible; otherwise returns \a false.*/
	bool isVisible() const { return lineVisible; }
	
	void setVisible(bool yes);
	
	/** Shorthand for \a setVisible(true). */
	void show() { setVisible(true); } 
	
	/** Returns the \a z index of the canvas item, which is used for visual order: 
      higher-z items obscure (are in front of) lower-z items. */
	double z() const { return lineZ; }
	
	void setZ(double z); 
	
	/** Returns a pointer to the ellips node. */
	QCanvasPolygonalItem* startNode(){ return firstNode; } 

	/** Returns a pointer to the rectangle node. */
	QCanvasPolygonalItem* endNode(){ return secondNode; } 
	
	void setArrowWidth(int width);
	void setArrowHeight(int height);
	void setStartArrow(bool yes);
	void setEndArrow(bool yes);
	
	void addBreakPoint(CSegment *s);
	void deleteBreakPoint(CHandle *h);
	void update();
	
	/** Returns the number of breakpoints. */
	int breaksCount() { return breaks; }
	
	QPoint breakpoint(int i);
  
  /** Returns a pointer to the \a i-th segment of the line. */
  CSegment* segment(int i) { return segments.at(i); }
  
  void moveBreakPoint(int i, int x, int y);
};





#endif








