/***************************************************************************
    copyright            : (C) 2004 - 2005 by Marcin Szpyrka
    email                : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "clinewithlabels.h"


//--------------------------------------------------------------------------
/** Adds the label \a lab to the list of the object labels and returns \a true
  or returns \a false if a label with the same tag already exists. */
  
bool CLineWithLabels::addLabel(CLabel* lab)
{
  int ltag = lab->tag();
  CLabel* i;

  // check whether such tag already exists
  for(i = labels.first(); i; i = labels.next())
  {
    if(i->tag() == ltag)
      return false;
  }

  labels.append(lab);
  return true;
}


//--------------------------------------------------------------------------
/**
  Creates a new label, adds it to the list of the object labels and returns \a true
  or returns \a false if a label with the same tag already exists.
  \param ltag the label tag,
  \param x the horizontal position of the label,
  \param y the horizontal position of the label,
  \param text the label text,
  \param yes if \a yes is \a true the label is placed inside a box
*/

bool CLineWithLabels::addLabel(int ltag, int x, int y, QString text, bool yes)
{
  CLabel* i;

  // check whether such tag already exists
  for(i = labels.first(); i; i = labels.next())
  {
    if(i->tag() == ltag)
      return false;
  }

  i = new CLabel(ltag, x, y, text, yes, canvas());
  labels.append(i);
  return true;
}


//--------------------------------------------------------------------------
/** Removes the label with \a ltag tag from the list of the object labels
  and returns \a true or returns \a false if there is no such a label. */

bool CLineWithLabels::removeLabel(int ltag)
{
  CLabel* i;

  // check whether such tag exists
  for(i = labels.first(); i; i = labels.next())
  {
    if(i->tag() == ltag)
    {
      labels.remove(i);
      return true;
    }
  }
  
  return false;
}


//--------------------------------------------------------------------------
/**	Removes the label \a lab from the list of the object labels
  and returns \a true or returns \a false if there is no such a label. */

bool CLineWithLabels::removeLabel(CLabel* lab)
{
  CLabel* i;

  // check whether such tag already exists
  for(i = labels.first(); i; i = labels.next())
  {
    if(i == lab)
    {
      labels.remove(i);
      return true;
    }
  }
  
  return false;
}


//--------------------------------------------------------------------------
/** Updates handles and arrows' heads. */
	
void CLineWithLabels::update()
{
	CLine::update();	
	
	// pocztkowe umieszczenie etykiety
	
	CLabel* label;
	label = this->label(0);
	QPoint p1, p2;
	p1 = points.point(0);
	p2 = points.point(1);
	
	int x1, x2, y1, y2;
	x1 = p1.x();
	x2 = p2.x();
	y1 = p1.y();
	y2 = p2.y();
	
	if((label->x() == 0) && (label->y() == 0))
	{
		x1 = (x1 + x2) / 2;
		y1 = (y1 + y2) / 2;
		
		label->setX(x1);
		label->setY(y1);
	}
}




