/***************************************************************************
    copyright            : (C) 2004 - 2005 by Marcin Szpyrka
    email                : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CLINEWITHLABELS_H
#define CLINEWITHLABELS_H

#include "cline.h"
#include "clabel.h"
#include <qptrlist.h>

/**
  The CLineWithLabels class provides a polylinee on a QCanvas
	with a list of labels connected with it.
*/

class CLineWithLabels : public CLine
{
protected:
	QPtrList<CLabel> labels;

public:
	enum { RTTI = 10010 };

  /** Creates a line with two characteristic points that connects two nodes. */
	CLineWithLabels(int ltag, QCanvas* canvas, QCanvasPolygonalItem *firstPtr,
									QCanvasPolygonalItem *secondPtr) : 
	                CLine(ltag, canvas, firstPtr, secondPtr)
    { labels.setAutoDelete(true); }

	/** Destroys the line. */
	~CLineWithLabels(){}

	/** Returns CLineWithLabels class RTTI */
	int rtti() const { return RTTI; }

  // adding labels
	bool addLabel(CLabel* lab);
	bool addLabel(int ltag, int x, int y, QString text, bool yes);

	// removing labels
	bool removeLabel(int ltag);
	bool removeLabel(CLabel* lab);

	// access to labels
	/**	Returns a pointer to the label at position \a index in the list,
  or 0 if the index is out of range. The valid range is 0..(labelsCount() - 1)
  inclusive. */
  CLabel* label(int index) { return labels.at(index); }

	/** Returns the number of labels. */
	int labelsCount() const { return labels.count(); };

  void update();
};

#endif












