/***************************************************************************
    copyright   : (C) 2004 - 2005 by Marcin Szpyrka, Piotr Matyasik
    email       : {mszpyrka,ptm]@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cnetview.h"
#include "crectanglewithlabels.h"
#include "cellipsewithlabels.h"
#include "clinewithlabels.h"
#include <qstring.h>
#include "../model/cplace.h"
#include "../model/ctransition.h"
#include "../model/carc.h"


//--------------------------------------------------------------------------
/** Constructs a NetView for \a canvas with parent widget \a parent and
  name \a name. */

CNetView::CNetView(QWidget * parent, QCanvas * canvas, CNet *model) :
  QCanvasView(parent, NULL, 0)
{
  // default values
  plWidth  = 50;
  trWidth  = 45;
  plHeight = 35;
  trHeight = 25;
  ahWidth  = 10;
  ahHeight = 15;
  linWidth = 1;
  net      = model;
  placeID  = transID = 1;
  viewMode = vmSelect;
  moved    = false;
  curHand  = 0;

  setMouseTracking(true);
  setCanvas(canvas);
  canvas->setAdvancePeriod(20);
  canvas->setBackgroundColor(QColor(255, 255, 255));
  canvas->resize(1512, 1512);
  show();

  // create actions
  addBreakPointAction = new QAction(tr("&Add Break Point"), 0, this);
  connect(addBreakPointAction, SIGNAL(activated()), this, SLOT(addBreakPoint()));

  deleteBreakPointAction = new QAction(tr("&Delete Break Point"), 0, this);
  connect(deleteBreakPointAction, SIGNAL(activated()), this, SLOT(deleteBreakPoint()));
}


//--------------------------------------------------------------------------
/** Reimplemeted function contentsMousePressEvent(). */

void CNetView::contentsMousePressEvent(QMouseEvent * e)
{
  QPoint pos = e->pos();
  int mouseX = e->x();
  int mouseY = e->y();
  mouseBufferX = e->x();
  mouseBufferY = e->y();
  QCanvasItemList clicked = canvas()->collisions(e->pos());
  QCanvasItemList items = canvas()->allItems();
  QCanvasItemList::iterator i;

  if((viewMode == vmSelect) && ((e->state() & ShiftButton) == 0))  
  {
    // klikniecie na pustym obszarze
    if(clicked.empty())
    {
      // ustawiamy wszystkie elementy jako niezaznaczone
      for (i = items.begin(); i != items.end(); i++)
      {
        (*i)->setSelected(false);
      }

      curHand = 0;
      arcPoints = 0;
      emit canvasSelected(this);
      return;
    }
    else // albo klikamy element by go zaznaczyc, albo mamy zamiar go ciagnac
    {
      curHand = 0;
      bool itemFound = false;

      for(i = clicked.begin(); i != clicked.end(); i++)
      {
        // jezeli klikniety zostal uchwyt, to zapamietujemy go
        if((*i)->rtti() == 10011) // line handler
        {
          if(CHandle *hand = static_cast<CHandle* >(*i))
          {
            curHand = hand;
          }
        }
        // zapamietujemy, czy klikniety zostal jakis zaznaczony element
        // jezeli tak, to prawdopodobnie bedzie on przeciagany
        if((*i)->isSelected())
        {
          itemFound = true;
        }
      }
      if(itemFound == false)
      {
        for (i = items.begin(); i != items.end(); i++)
        {
          (*i)->setSelected(false);
        }
      }
    }
    return;
  }

  // add place
  if((viewMode == vmAddPlace) && (e->button() == LeftButton))
  {
    // create the model place
    QString nazwa("p");
    nazwa = nazwa + QString::number(placeID);
    CPlace *place = new CPlace(placeID, nazwa, 0, 0);
    net->addPlace(place);

    // create the place view (ellipse)
    ellipse = new CEllipseWithLabels(placeID, mouseX, mouseY, plWidth, plHeight, this->canvas());
    ellipse->setPen(QPen(Qt::black, linWidth));
    ellipse->setBrush(Qt::white);
    ellipse->setModelPointer(place);
    placeID++;
    
    // place name label
    ellipse->addLabel(0, mouseX + (plWidth / 2) + 20, mouseY - (plHeight / 2) - 5, nazwa, false);
    ellipse->label(0)->setZ(3);
    ellipse->setZ(1);
    ellipse->setDots(0);
    ellipse->show();

    for (i = items.begin(); i != items.end(); i++)
    {
      (*i)->setSelected(false);
    }
    ellipse->setSelected(true);
    emit placeSelected(ellipse);
    this->canvas()->update();
    return;
  }

  // add transition
  if((viewMode == vmAddTrans) && (e->button() == LeftButton))
  {
    // create the model transition
    QString nazwa("t");
    nazwa = nazwa + QString::number(transID);
    CTransition *trans = new CTransition(transID, nazwa, 0);
    net->addTransition(trans);
    
    // create the view transition (rectangle)
    rectangle = new CRectangleWithLabels(transID, mouseX, mouseY, trWidth, trHeight, this->canvas());
    rectangle->setPen(QPen(Qt::black, linWidth));
    rectangle->setBrush(Qt::white);
    rectangle->setModelPointer(trans);
    transID++;
    
    // transition name label
    rectangle->addLabel(0, mouseX + (trWidth / 2) + 20, mouseY - (trHeight / 2) - 5, nazwa, false);
    // transition time|priority label
    rectangle->addLabel(1, mouseX + (trWidth / 2) + 20, mouseY - (trHeight / 2) + 30, "0|0", true);     
    rectangle->label(0)->setZ(3);
    rectangle->label(1)->setZ(3);
    rectangle->setZ(1);
    rectangle->show();
    rectangle->label(1)->setVisible(false);

    for (i = items.begin(); i != items.end(); i++)
    {
      (*i)->setSelected(false);
    }
    rectangle->setSelected(true);
    emit transSelected(rectangle);
    this->canvas()->update();
    return;
  }

  // select the first node for an arc
  if((viewMode == vmAddArc) && (arcPoints == 0)) 
  {
    if (!clicked.empty()) 
    {
      QCanvasItem* element = clicked.first();;
      if(element->rtti() == 10005) // CEllipseWithLabels
      {
        ellipse = static_cast<CEllipseWithLabels* >(element);
        arcPoints = -1;
        mouseBufferX = mouseX;
        mouseBufferY = mouseY;
      }
      else if(element->rtti() == 10004) // CRectangleWithLabels
      {
        rectangle = static_cast<CRectangleWithLabels* >(element);
        arcPoints = 1;
        mouseBufferX = mouseX;
        mouseBufferY = mouseY;
      }
    }
    return;
  }
  
  if((viewMode == vmInteractiveRun) && (e->button() == RightButton))
  {
    i = clicked.begin();
    if((*i)->rtti() == 10004) // rectangle with labels
    {
      rectangle = static_cast<CRectangleWithLabels* >(*i);
      if(rectangle->isEnabled())
      {
        CTransition *trans = static_cast<CTransition* >(rectangle->modelPointer());
        net->checkOff(trans, !(rectangle->isActive()));
        rectangle->setActive(!(rectangle->isActive()));
      }
    }
    
    // update view
    for(i = items.begin(); i != items.end(); i++)
    {
      (*i)->setSelected(false);

      if((*i)->rtti() == 10004) // rectangle with labels
      {
        rectangle = static_cast<CRectangleWithLabels*>(*i);
        CTransition *trans = static_cast<CTransition*>(rectangle->modelPointer());
        rectangle->setEnabled(net->enable(trans));
      }     
    }
  }  

  if((viewMode == vmInteractiveRun) && (e->button() == LeftButton))
  {
    i = clicked.begin();
    if((*i)->rtti() == 10004) // rectangle with labels
    {
      rectangle = static_cast<CRectangleWithLabels* >(*i);
      if(rectangle->isEnabled())
      {
        CTransition *trans = static_cast<CTransition* >(rectangle->modelPointer());
        net->checkOff(trans, true);
        net->fire();
        emit updateClock(net->clock());
      }
    }

    // update view
    int wt;
    QString description;
    for(i = items.begin(); i != items.end(); i++)
    {
      (*i)->setSelected(false);

      if((*i)->rtti() == 10004) // rectangle with labels
      {
        rectangle = static_cast<CRectangleWithLabels*>(*i);
        CTransition *trans = static_cast<CTransition*>(rectangle->modelPointer());
        rectangle->setEnabled(net->enable(trans)); 
        rectangle->setActive(false);
        
        if((!withPriorities) && (withTime))
        {
          wt = trans->waitingTime();
          if(wt < 0) description = "-";
          else description = QString("%1").arg(trans->waitingTime());
          rectangle->label(1)->setText(description);
        }
        else if((withPriorities) && (withTime))
        {
          wt = trans->waitingTime();
          if(wt < 0) description = QString("%1|-").arg(trans->priority());
          else description = QString("%1|%2").arg(trans->priority()).arg(trans->waitingTime());
          rectangle->label(1)->setText(description);
        } 
      }
      if((*i)->rtti() == 10005)
      {
        ellipse = static_cast<CEllipseWithLabels* >(*i);
        CPlace *place = static_cast<CPlace* >(ellipse->modelPointer());
        ellipse->setDots(place->marking());
      }      
    }
  }
}


//--------------------------------------------------------------------------
/** Reimplemeted function contentsMouseReleaseEvent(). */

void CNetView::contentsMouseReleaseEvent(QMouseEvent * e)
{
  QCanvasItemList clicked = canvas()->collisions(e->pos());
  QCanvasItemList items = canvas()->allItems();
  QCanvasItemList::iterator i;
  CLabel* label;

  if((viewMode == vmSelect))
  {
    if(clicked.empty())
    {
      for (i = items.begin(); i != items.end(); i++)
      {
        (*i)->setSelected(false);
      }
      emit canvasSelected(this);
    }
    else if(!moved)
    {
      if(((e->state() & ShiftButton) == 0) && (viewMode == vmSelect))
      {
        for(i = items.begin(); i != items.end(); i++)
        {
          (*i)->setSelected(false);
        }
      }

      i = clicked.begin();
      (*i)->setSelected(!(*i)->isSelected()); 
      int rtti=((QCanvasItem*)(*i))->rtti();

      if(rtti == 10005) // place
      {
        ellipse = static_cast<CEllipseWithLabels*>(*i);
        emit placeSelected(ellipse);
      }
      else if(rtti == 10004) // transition
      {       
        rectangle = static_cast<CRectangleWithLabels*>(*i);
        emit transSelected(rectangle);
      }
      else if(rtti == 10003) // label
      {
        // odnajdujemy odpowiedni element i wyswietlamy jego wlasnosci
        CLabel* clickedLabel = static_cast<CLabel*>(*i);
        
        QCanvasItemList::iterator j;
        bool found = false;
        j = items.begin();

        while((!found) && (j != items.end()))
        {
          if((*j)->rtti() == 10005)
          {
            ellipse = static_cast<CEllipseWithLabels*>(*j);
            int lCount = ellipse->labelsCount();
            for(int k = 0; k < lCount; k++)
            {
              label = ellipse->label(k);
              if(label == clickedLabel)
              {
                found = true;
                emit placeSelected(ellipse);
              }
            }
          }
          if((*j)->rtti() == 10004) // rectangle 
          { 
            rectangle = static_cast<CRectangleWithLabels*>(*j);
            int lCount = rectangle->labelsCount();
            for(int k = 0; k < lCount; k++)
            {
              label = rectangle->label(k);
              if(label == clickedLabel)
              {
                found = true;
                emit transSelected(rectangle);
              }
            }
          }
          if((*j)->rtti() == 10013) // CSegment
          {
            CSegment* s = static_cast<CSegment*>(*j);
            line = static_cast<CLineWithLabels*>(s->owner());
            int lCount = line->labelsCount();
            for(int k = 0; k < lCount; k++)
            {
              label = line->label(k);
              if(label == clickedLabel)
              {
                found = true;
                emit arcSelected(line);
              }
            }              
          }
          
          j++;
        }
      }  // end label 
      else if (rtti == 10013)  // CSegment
      {
        CSegment* s = static_cast<CSegment*>(*i);
        line = static_cast<CLineWithLabels*>(s->owner());
        emit arcSelected(line);
      }
    }
    update();
  }

  // select the second node for an arc going from a transition to a place
  // and create the arc
  else if((viewMode == vmAddArc) && (arcPoints == 1)) 
  {
    if (!clicked.empty()) 
    {
      QCanvasItem* element = clicked.first();
      
      if(element->rtti() == 10005)  // CEllipseWithLabels
      {
        ellipse = static_cast<CEllipseWithLabels* >(element);
        CPlace* place = static_cast<CPlace* >(ellipse->modelPointer());
        CTransition* trans = static_cast<CTransition*>(rectangle->modelPointer());
        CArc* arc;

        // check whether such an arc already exists
        int j = net->arcsCount();

        for(int k = 0; k < j; k++)
        {
          arc = net->arc(k);
          if((arc->place() == place)
            && (arc->transition() == trans)
            && (arc->direction() == 1))
          {
            return;
          }
        }

        // create the view line
        line = new CLineWithLabels(0, canvas(), ellipse, rectangle);
        arc = new CArc(0, trans, place);
        line->setModelPointer(arc);
        net->addArc(arc);
        
        line->addLabel(0, 0, 0, "1", false);
        
        // set graphics attributes and show the line
        line->setStartArrow(true);
        line->setEndArrow(false);
        line->setPen(QPen(Qt::black,linWidth));
        line->setBrush(Qt::black);
        line->setArrowWidth(ahWidth);
        line->setArrowHeight(ahHeight);
        line->label(0)->setZ(3);
        line->setZ(2);
        line->show();
        line->update();
        line->label(0)->hide();

        for (i = items.begin(); i != items.end(); i++) 
        {
          (*i)->setSelected(false);
        }
        line->setSelected(true);
        emit arcSelected(line);
        this->canvas()->update();
      }
    }

    arcPoints = 0;
    update();
    return;
  }

  // select the second node for an arc going from a place to a transition
  // and create the arc
  else if((viewMode == vmAddArc) && (arcPoints == -1)) 
  {
    if (!clicked.empty()) 
    {
      QCanvasItem* element = clicked.first();
      
      if(element->rtti() == 10004)  // CRectangleWithLabels
      {
        rectangle = static_cast<CRectangleWithLabels* >(element);
        CPlace* place = static_cast<CPlace* >(ellipse->modelPointer());
        CTransition* trans = static_cast<CTransition*>(rectangle->modelPointer());
        CArc* arc;

        // check whether such an arc already exists
        int j = net->arcsCount();

        for(int k = 0; k < j; k++)
        {
          arc = net->arc(k);
          if((arc->place() == place)
            && (arc->transition() == trans)
            && (arc->direction() == 0))
          {
            return;
          }
        }

        // create the view line
        line = new CLineWithLabels(0, canvas(), ellipse, rectangle);
        arc = new CArc(0, place, trans);
        line->setModelPointer(arc);
        net->addArc(arc);
        
        line->addLabel(0, 0, 0, "1", false);
        
        // set graphics attributes and show the line
        line->setStartArrow(false);
        line->setEndArrow(true);
        line->setPen(QPen(Qt::black,linWidth));
        line->setBrush(Qt::black);
        line->setArrowWidth(ahWidth);
        line->setArrowHeight(ahHeight);
        line->label(0)->setZ(3);
        line->setZ(2);
        line->show();
        line->update();
        line->label(0)->hide();

        for (i = items.begin(); i != items.end(); i++) 
        {
          (*i)->setSelected(false);
        }
        line->setSelected(true);
        emit arcSelected(line);
        this->canvas()->update();
      }
    }  
      
    arcPoints = 0;
    update();
    return;
  }

  moved = 0;
}


//--------------------------------------------------------------------------
/** Reimplemeted function contentsMouseMoveEvent(). */

void CNetView::contentsMouseMoveEvent(QMouseEvent * e)
{
  if (((e->state() & LeftButton) != 0) && (viewMode == vmSelect))
  {
    moved = 1;
    int dx = e->x() - mouseBufferX;
    int dy = e->y() - mouseBufferY;
    mouseBufferX = e->x();
    mouseBufferY = e->y();
    QCanvasItemList items = canvas()->allItems();
    QCanvasItemList clicked = canvas()->collisions(e->pos());
    QCanvasItemList::iterator i;

    for(i = items.begin(); i != items.end(); i++)
    {
      if(((*i)->isSelected()) && ((*i)->rtti() != 10013) // line segment
        && ((*i)->rtti() != 10012)) // arrowhead
      {
        (*i)->moveBy(dx, dy);
      }

      // update all arcs
      if((*i)->rtti() == 10013) // line segment
      {
        CSegment* s;
        if((s = (CSegment*)(*i)))
        {
          CLine *line;
          if((line = (CLine*)(s->owner())))
          {
            line->update();
          }
        }
      }
    }
    if(curHand && curHand->tag() != 0)
    {
      curHand->move(e->x(), e->y());
    }

    canvas()->update();
  }
}


//--------------------------------------------------------------------------
/** Sets the net view mode to \a m. */

void CNetView::setMode(CNetViewMode m)
{
  viewMode = m;
}


//--------------------------------------------------------------------------
/** Switches the net view to select mode. */

void CNetView::setSelectMode()
{
  arcPoints = 0;
  viewMode = vmSelect;
}


//--------------------------------------------------------------------------
/** Switches the net view to add place mode. */

void CNetView::setAddPlaceMode()
{
  QCanvasItemList items = canvas()->allItems();
  for (QCanvasItemList::iterator i = items.begin(); i != items.end(); i++)
  {
    (*i)->setSelected(false);
  }
  emit canvasSelected(this);

  arcPoints = 0;
  viewMode = vmAddPlace;
}


//--------------------------------------------------------------------------
/** Switches the net view to add transition mode. */

void CNetView::setAddTransMode()
{
  QCanvasItemList items = canvas()->allItems();
  for (QCanvasItemList::iterator i = items.begin(); i != items.end(); i++)
  {
    (*i)->setSelected(false);
  }
  emit canvasSelected(this);

  arcPoints = 0;
  viewMode = vmAddTrans;
}


//--------------------------------------------------------------------------
/** Switches the net view to add arc mode. */

void CNetView::setAddArcMode()
{
  QCanvasItemList items = canvas()->allItems();
  for (QCanvasItemList::iterator i = items.begin(); i != items.end(); i++)
  {
    (*i)->setSelected(false);
  }
  emit canvasSelected(this);

  arcPoints = 0;
  viewMode = vmAddArc;
}


//---------- void CNetView::setAutomaticRunMode()  -------------------------
/** Switches the net view to automatic run mode. */

void CNetView::setAutomaticRunMode()
{
  setMode(vmAutomaticRun);

  net->startSimulation();
  emit updateClock(net->clock());
 
  CRectangleWithLabels* rect;
  CTransition*          trans;

  QCanvasItemList items = canvas()->allItems();
  for (QCanvasItemList::iterator i = items.begin(); i != items.end(); i++)
  {
    (*i)->setSelected(false);

    if((*i)->rtti() == 10004) // rectangle with labels
    {
      rect = static_cast<CRectangleWithLabels*>(*i);
      trans = static_cast<CTransition*>(rect->modelPointer());
      QString description;
      int wt;
      
      rect->setEnabled(net->enable(trans));
      
      if((!withPriorities) && (!withTime))
      {
        rect->label(1)->setVisible(false);
      }
      else if((withPriorities) && (!withTime))
      {
        description = QString("%1").arg(trans->priority());
        rect->label(1)->setText(description);        
      }
      else if((!withPriorities) && (withTime))
      {
        wt = trans->waitingTime();
        if(wt < 0) description = "-";
        else description = QString("%1").arg(trans->waitingTime());
        rect->label(1)->setText(description);
      }
      else if((withPriorities) && (withTime))
      {
        wt = trans->waitingTime();
        if(wt < 0) description = QString("%1|-").arg(trans->priority());
        else description = QString("%1|%2").arg(trans->priority()).arg(trans->waitingTime());
        rect->label(1)->setText(description);
      }
    }
  }
  
  // to do
  
  
}


//---------- void CNetView::setInteractiveRunMode()  -----------------------
/** Switches the net view to interactive run mode. */

void CNetView::setInteractiveRunMode()
{
  setMode(vmInteractiveRun);
  net->startSimulation();
  emit updateClock(net->clock());
 
  CRectangleWithLabels* rect;
  CTransition*          trans;

  QCanvasItemList items = canvas()->allItems();
  for (QCanvasItemList::iterator i = items.begin(); i != items.end(); i++)
  {
    (*i)->setSelected(false);

    if((*i)->rtti() == 10004) // rectangle with labels
    {
      rect = static_cast<CRectangleWithLabels*>(*i);
      trans = static_cast<CTransition*>(rect->modelPointer());
      QString description;
      int wt;
      
      rect->setEnabled(net->enable(trans));
      
      if((!withPriorities) && (!withTime))
      {
        rect->label(1)->setVisible(false);
      }
      else if((withPriorities) && (!withTime))
      {
        description = QString("%1").arg(trans->priority());
        rect->label(1)->setText(description);        
      }
      else if((!withPriorities) && (withTime))
      {
        wt = trans->waitingTime();
        if(wt < 0) description = "-";
        else description = QString("%1").arg(trans->waitingTime());
        rect->label(1)->setText(description);
      }
      else if((withPriorities) && (withTime))
      {
        wt = trans->waitingTime();
        if(wt < 0) description = QString("%1|-").arg(trans->priority());
        else description = QString("%1|%2").arg(trans->priority()).arg(trans->waitingTime());
        rect->label(1)->setText(description);
      }
    }
  }  
}


//--------------------------------------------------------------------------
/** Stops simulation but the net view still remains in a simulation mode. */

void CNetView::stopSimulation()
{
  if(viewMode == vmStopped) return;
  
  setMode(vmStopped);
  net->stopSimulation();

  CRectangleWithLabels* rect;
  CTransition *trans;
  QCanvasItemList items = canvas()->allItems();

  for (QCanvasItemList::iterator i = items.begin(); i != items.end(); i++)
  {
    (*i)->setSelected(false);

    if((*i)->rtti() == 10004) // rectangle with labels
    {
      rect = static_cast<CRectangleWithLabels*>(*i);
      rect->setEnabled(false);
      rect->setActive(false);
      trans = static_cast<CTransition*>(rect->modelPointer());
      QString description = QString("%1|%2").arg(trans->priority()).arg(trans->time());
      rect->label(1)->setText(description);
      
      if(description == "0|0")
      {
        rect->label(1)->setVisible(false);
      }
      else
      {
        rect->label(1)->setVisible(true);
      }      
    }
  }
}


//--------------------------------------------------------------------------
/** Resets the initial marking. */

void CNetView::resetInitialMarking()
{
  QCanvasItemList items = canvas()->allItems();
  QCanvasItemList::iterator i;

  for(i = items.begin(); i != items.end(); i++)
  {
    if((*i)->rtti() == 10005)
    {
      ellipse = static_cast<CEllipseWithLabels* >(*i);
      CPlace *place = static_cast<CPlace* >(ellipse->modelPointer());
      place->setMarking(place->initialMarking());
      ellipse->setDots(place->marking());
    }
  }

  if(viewMode == vmInteractiveRun)
  {
    for(i = items.begin(); i != items.end(); i++)
    {
      (*i)->setSelected(false);

      if((*i)->rtti() == 10004) // rectangle with labels
      {
        rectangle = static_cast<CRectangleWithLabels*>(*i);
        CTransition *trans = static_cast<CTransition*>(rectangle->modelPointer());
        if(net->enable(trans)) 
        {
          rectangle->setEnabled(true);
        }  
        else 
        {
          rectangle->setEnabled(false);
        }
      }
    }
  }
}


//--------------------------------------------------------------------------
/** Lay out selected elements to the left. */

void CNetView::alingmentLeft()
{
  double nx, xe, xr, xp; // xe, xr - They are used as a correction because
                         // ellipses and rectangles have different widths.
  QCanvasItemList items = canvas()->allItems();
  QCanvasItemList selItems;
  QCanvasItemList::iterator a, a2;
  CSegment* s;
  CLine *line;

  for(a = items.begin(); a != items.end(); a++)
  {
    if(((*a)->isSelected())
      && ((*a)->rtti()==10004 || (*a)->rtti()==10005)) // 10004 rect, 10005 ellipse
    {
      selItems.append(*a);
    }
  }

  if(!selItems.isEmpty())
  {
    a2 = selItems.begin();
    for(a = selItems.begin(); a != selItems.end(); a++)
    {
      if((*a)->x() < (*a2)->x())
      {
        a2 = a;
      }
    }

    nx = (*a2)->x();
    xp = (plWidth - trWidth)/2;

    if((*a2)->rtti()==10005) // ellipse
    {
      xe = 0;
      xr = xp;
    }
    else
    {
      xr = 0;
      xe = -xp;
    }

    for(a = selItems.begin(); a != selItems.end(); a++)
    {
      if((a != a2) && ((*a)->rtti()==10005)) 
      {
        (*a)->setX(nx + xe);
      }
      if((a != a2) && ((*a)->rtti()==10004))
      {
        (*a)->setX(nx + xr);
      }
    }
  }

  // update all arcs
  for(a = items.begin(); a != items.end(); a++)
  {
    if((*a)->rtti() == 10013) // line segment
    {
      if((s = (CSegment*)(*a)))
      {
        if((line = (CLine*)(s->owner())))
        {
          line->update();
        }
      }
    }
  }
  canvas()->update();
}


//--------------------------------------------------------------------------
/** Lay out selected elements to the right. */

void CNetView::alingmentRight()
{
  double nx, xe, xr, xp; // xe, xr - They are used as a correction because
                         // ellipses and rectangles have different widths.�
  QCanvasItemList items = canvas()->allItems();
  QCanvasItemList selItems;
  QCanvasItemList::iterator a, a2;
  CSegment* s;
  CLine *line;

  for(a = items.begin(); a != items.end(); a++)
  {
    if(((*a)->isSelected())
      && ((*a)->rtti()==10004 || (*a)->rtti()==10005)) 
    {
      selItems.append(*a);
    }
  }

  if(!selItems.isEmpty())
  {
    a2 = selItems.begin();
    for(a = selItems.begin(); a != selItems.end(); a++)
    {
      if((*a)->x() > (*a2)->x())
      {
        a2 = a;
      }
    }

    nx = (*a2)->x();
    xp = (plWidth - trWidth)/2;

    if((*a2)->rtti()==10005) 
    {
      xe = 0;
      xr = xp;
    }
    else
    {
      xr = 0;
      xe = -xp;
    }

    for(a = selItems.begin(); a != selItems.end(); a++)
    {
      if((a != a2) && ((*a)->rtti()==10005)) 
      {
        (*a)->setX(nx + xe);
      }
      if((a != a2) && ((*a)->rtti()==10004))
      {
        (*a)->setX(nx + xr);
      }
    }
  }


  // update all arcs
  for(a = items.begin(); a != items.end(); a++)
  {
    if((*a)->rtti() == 10013) // line segment
    {
      if((s = (CSegment*)(*a)))
      {
        if((line = (CLine*)(s->owner())))
        {
          line->update();
        }
      }
    }
  }
  canvas()->update();
}


//--------------------------------------------------------------------------
/** Lay out selected elements to the top. */

void CNetView::alingmentTop()
{
  double ny, ye, yr, yp; // ye, yr - They are used as a correction because
                         // ellipses and rectangles have different heights
  QCanvasItemList items = canvas()->allItems();
  QCanvasItemList selItems;
  QCanvasItemList::iterator a, a2;
  CSegment* s;
  CLine *line;

  for(a = items.begin(); a != items.end(); a++)
  {
    if(((*a)->isSelected())
      && ((*a)->rtti()==10004 || (*a)->rtti()==10005)) 
    {
      selItems.append(*a);
    }
  }

  if(!selItems.isEmpty())
  {
    a2 = selItems.begin();
    for(a = selItems.begin(); a != selItems.end(); a++)
    {
      if((*a)->y() < (*a2)->y())
      {
        a2 = a;
      }
    }

    ny = (*a2)->y();
    yp = (plHeight - trHeight)/2;

    if((*a2)->rtti()==10005) // ellipse
    {
      ye = 0;
      yr = yp;
    }
    else
    {
      yr = 0;
      ye = -yp;
    }

    for(a = selItems.begin(); a != selItems.end(); a++)
    {
      if((a != a2) && ((*a)->rtti()==10005)) 
      {
        (*a)->setY(ny + ye);
      }
      if((a != a2) && ((*a)->rtti()==10004))
      {
        (*a)->setY(ny + yr);
      }
    }
  }

  // update all arcs
  for(a = items.begin(); a != items.end(); a++)
  {
    if((*a)->rtti() == 10013) // line segment
    {
      if((s = (CSegment*)(*a)))
      {
        if((line = (CLine*)(s->owner())))
        {
          line->update();
        }
      }
    }
  }
  canvas()->update();
}


//--------------------------------------------------------------------------
/** Lay out selected elements to the bottom. */

void CNetView::alingmentBottom()
{
  double ny, ye, yr, yp; // ye, yr - They are used as a correction because
                         // ellipses and rectangles have different heights
  QCanvasItemList items = canvas()->allItems();
  QCanvasItemList selItems;
  QCanvasItemList::iterator a, a2;
  CSegment* s;
  CLine *line;

  for(a = items.begin(); a != items.end(); a++)
  {
    if(((*a)->isSelected())
      && ((*a)->rtti()==10004 || (*a)->rtti()==10005)) 
    {
      selItems.append(*a);
    }
  }

  if(!selItems.isEmpty())
  {
    a2 = selItems.begin();
    for(a = selItems.begin(); a != selItems.end(); a++)
    {
      if((*a)->y() > (*a2)->y())
      {
        a2 = a;
      }
    }

    ny = (*a2)->y();
    yp = (plHeight - trHeight)/2;

    if((*a2)->rtti()==10005) // 
    {
      ye = 0;
      yr = yp;
    }
    else
    {
      yr = 0;
      ye = -yp;
    }

    for(a = selItems.begin(); a != selItems.end(); a++)
    {
      if((a != a2) && ((*a)->rtti()==10005)) 
      {
        (*a)->setY(ny + ye);
      }
      if((a != a2) && ((*a)->rtti()==10004))
      {
        (*a)->setY(ny + yr);
      }
    }
  }

  // update all arcs
  for(a = items.begin(); a != items.end(); a++)
  {
    if((*a)->rtti() == 10013) // line segment
    {
      if((s = (CSegment*)(*a)))
      {
        if((line = (CLine*)(s->owner())))
        {
          line->update();
        }
      }
    }
  }
  canvas()->update();
}


//--------------------------------------------------------------------------
/** Perform horizontal spread of selected elements. */

void CNetView::alingmentHorizontalSpread()
{
  // TODO
}


//--------------------------------------------------------------------------
/** Perform vertical spread of selected elements. */

void CNetView::alingmentVerticalSpread()
{
  // TODO
}


//--------------------------------------------------------------------------
/** Undo the last action. */

void CNetView::undo()
{
  // TODO
}


//--------------------------------------------------------------------------
/** Redo the last action. */

void CNetView::redo()
{
  // TODO
}


//--------------------------------------------------------------------------
/** Cut selected elements to the clipboard. */

void CNetView::cut()
{
  // TODO
}


//--------------------------------------------------------------------------
/** Copy selected elements to the clipboard. */

void CNetView::copy()
{
  // TODO
}


//--------------------------------------------------------------------------
/** Paste elements from the clipboard. */

void CNetView::paste()
{
  // TODO
}


//--------------------------------------------------------------------------
/** Deletes selected elements. */

void CNetView::remove()
{
  CTransition *trans;
  CPlace *place;
  CArc *arc;

  int rtti, rtti2;
  QCanvasItemList items = canvas()->allItems();
  QCanvasItemList items2;
  QCanvasItemList::iterator a;
  QCanvasItemList::iterator a2;
  CSegment *segment;

  a = items.begin();
  while(a != items.end())
  {
    if ((*a)->isSelected())
    {
      rtti = ((QCanvasItem*)(*a))->rtti();

      if(rtti == 10005)  // place
      {
        ellipse = static_cast<CEllipseWithLabels* >(*a);

        // remove arcs connected with this place
        items2 = canvas()->allItems();
        a2 = items2.begin();
        while(a2 != items2.end())
        {
          rtti2 = ((QCanvasItem*)(*a2))->rtti();
          if(rtti2 == 10013)  // segment
          {
            segment = static_cast<CSegment* >(*a2);
            line = static_cast<CLineWithLabels* >(segment->owner());
            CEllipseWithLabels* startEllipse;
            startEllipse = static_cast<CEllipseWithLabels*>(line->startNode());
            
            if(startEllipse == ellipse)
            {
              arc = static_cast<CArc* >(line->modelPointer());
              net->removeArc(arc);
              delete line;
              items2 = canvas()->allItems();
              a2 = items2.begin();
            }
            else
            {
              a2++;
            }
          }
          else
          {
            a2++;
          }
        }

        place = static_cast<CPlace* >(ellipse->modelPointer());
        net->removePlace(place);
        ellipse->removeAllLabels();
        delete ellipse;
        items = canvas()->allItems();
        a = items.begin();
      }

      if(rtti == 10004)  // transition
      {
        rectangle = static_cast<CRectangleWithLabels* >(*a);

        // remove arcs connected with this transition
        items2 = canvas()->allItems();
        a2 = items2.begin();
        while(a2 != items2.end())
        {
          rtti2 = ((QCanvasItem*)(*a2))->rtti();
          if(rtti2 == 10013)  // segment
          {
            segment = static_cast<CSegment* >(*a2);
            line = static_cast<CLineWithLabels* >(segment->owner());
            if(line->endNode() == rectangle)
            {
              arc = static_cast<CArc* >(line->modelPointer());
              net->removeArc(arc);
              delete line;
              items2 = canvas()->allItems();
              a2 = items2.begin();
            }
            else
            {
              a2++;
            }
          }
          else
          {
            a2++;
          }
        }

        trans = static_cast<CTransition* >(rectangle->modelPointer());
        net->removeTransition(trans);
        rectangle->removeAllLabels();
        delete rectangle;
        items = canvas()->allItems();
        a = items.begin();
      }

      if(rtti == 10013)  // segment
      {
        segment = static_cast<CSegment* >(*a);
        line = static_cast<CLineWithLabels* >(segment->owner());
        if(line->isSelected())
        {
          arc = static_cast<CArc* >(line->modelPointer());
          net->removeArc(arc);
          delete line;
          items = canvas()->allItems();
          a = items.begin();
        }
        else
        {
          a++;
        }
      }
      else
      {
        a++;
      }
    }
    else
    {
      a++;
    }
  }
  canvas()->update();
}


//--------------------------------------------------------------------------
/** Deletes all elements. */

void CNetView::removeAll()
{
  selectAll();
  remove();
  placeID = transID = 1;
}


//--------------------------------------------------------------------------
/** Selects all elements. */

void CNetView::selectAll()
{
  QCanvasItemList items = canvas()->allItems();
  CSegment *segment;

 // if ((viewMode == vmSelect) )
  {
    for (QCanvasItemList::iterator i = items.begin(); i != items.end(); i++)
    {
      if(((*i)->rtti()==10004) || ((*i)->rtti()==10005))
      {
        (*i)->setSelected(true);
      }
      if((*i)->rtti()==10013)
      {
        segment = static_cast<CSegment* >(*i);
        line = static_cast<CLineWithLabels* >(segment->owner());
        line->setSelected(true);
      }
    }
  }
  emit canvasSelected(this);
}


//--------------------------------------------------------------------------
/** Set canvas size. */

void CNetView::setCanvasProperties(QString n, int x, int y)
{
  netName = n;
  canvas()->resize(x,y);
}


//--------------------------------------------------------------------------
/** Set place properties. */

void CNetView::setPlaceProperties(QString name, int capacity, int marking, int tag)
{
  CPlace* place = static_cast<CPlace* >(ellipse->modelPointer());
  
  place->setName(name);
  place->setCapacity(capacity);
  place->setInitialMarking(marking);
  place->setMarking(marking);
  place->setTag(tag);
  ellipse->setTag(tag);
  if(tag >= placeID) placeID = tag + 1;
  ellipse->setDots(marking);
  ellipse->label(0)->setText(name);
}


//--------------------------------------------------------------------------
/** Set transition properties */

void CNetView::setTransProperties(QString name, int priority, int time, int tag)
{
  CTransition* trans = static_cast<CTransition*>(rectangle->modelPointer());
  
  trans->setName(name);
  trans->setPriority(priority);
  trans->setTime(time);
  trans->setTag(tag);
  rectangle->setTag(tag);
  if(tag >= transID) transID = tag + 1;
  rectangle->label(0)->setText(name);
  QString description = QString("%1|%2").arg(priority).arg(time);
  rectangle->label(1)->setText(description);
  
  if((priority == 0) && (time == 0))
  {
    rectangle->label(1)->setVisible(false);
  }
  else
  {
    rectangle->label(1)->setVisible(true);
  }
}


//--------------------------------------------------------------------------
/** Set arc properties. */

void CNetView::setArcProperties(int weight)
{
  CArc* arc = static_cast<CArc*>(line->modelPointer());
  
  arc->setWeight(weight);
  line->label(0)->setText(QString("%1").arg(weight));

  if(weight > 1)
  {
    line->label(0)->setVisible(true);
  }
  else
  {
    line->label(0)->setVisible(false);
  }
}


//--------------------------------------------------------------------------
/** Creates a popup menu. */

void CNetView::contentsContextMenuEvent(QContextMenuEvent *e)
{
  QPopupMenu contextMenu(this);
  
  QCanvasItemList clicked = canvas()->collisions(e->pos());
  QCanvasItemList::iterator i;
  
  // firstly, we look for a handle
  for (i = clicked.begin(); i != clicked.end(); i++)
  {
    if(((*i)->rtti() == 10011) && ((*i)->isVisible())) // CHandle
    {
      CHandle* handle = static_cast<CHandle* >(*i);
      
      if(handle->tag() == 1)
      {
        curHand = handle;
        
        addBreakPointAction->addTo(&contextMenu);
        addBreakPointAction->setEnabled(false);
        deleteBreakPointAction->addTo(&contextMenu);
        deleteBreakPointAction->setEnabled(true);
        contextMenu.exec(e->globalPos());
        return;
      }
    }
  }

  for (i = clicked.begin(); i != clicked.end(); i++)
  {
    if(((*i)->rtti() == 10013) && ((*i)->isSelected())) // CSegment
    {
      addBreakPointAction->addTo(&contextMenu);
      addBreakPointAction->setEnabled(true);
      deleteBreakPointAction->addTo(&contextMenu);
      deleteBreakPointAction->setEnabled(false);
      contextMenu.exec(e->globalPos());
      return;
    }
  }  
}


//--------------------------------------------------------------------------
/** Adds a break point to the selected line. */

void CNetView::addBreakPoint()
{
  QCanvasItemList items = canvas()->allItems();
  QCanvasItemList::iterator i = items.begin();
  bool found = false;

  for (; i != items.end(); i++)
  {
    if(((*i)->rtti() == 10013) && ((*i)->isSelected()))
    {
      found = true;
      break;
    }
  }

  if(found)
  {
    CSegment* s;
    if((s = (CSegment*)(*i)))
    {
      if((line = (CLineWithLabels*)(s->owner())))
      {
        line->addBreakPoint(s);
      }
    }
  }
  canvas()->update();
}


//--------------------------------------------------------------------------
/** Deletes a break point from the selected line. */

void CNetView::deleteBreakPoint()
{
  line = static_cast<CLineWithLabels*>(curHand->owner());
  
  if(line)
  {
    line->deleteBreakPoint(curHand);
    curHand = 0;
  }
}


//--------------------------------------------------------------------------
/** Updates elements shapes after graphical options changed. */

void CNetView::updateElements()
{
  QCanvasItemList items = canvas()->allItems();
  QCanvasItemList::iterator i;
  CSegment* segment;
  
  for(i = items.begin(); i != items.end(); i++)
  {
    if((*i)->rtti() == 10005)
    {
      ellipse = static_cast<CEllipseWithLabels*>(*i);
      if(ellipse)
      {
        ellipse->setWidth(plWidth);
        ellipse->setHeight(plHeight);
        ellipse->setPen(QPen(Qt::black, linWidth));
      }
    }
    
    if((*i)->rtti() == 10004)
    {
      rectangle = static_cast<CRectangleWithLabels*>(*i);
      if(rectangle)
      {
        rectangle->setSize(trWidth, trHeight);
        rectangle->setPen(QPen(Qt::black, linWidth));
      }  
    }
    
    if((*i)->rtti() == 10013)
    {
      segment = static_cast<CSegment*>(*i);
      if(segment)
      {
        segment->setPen(QPen(Qt::black, linWidth));
        line = static_cast<CLineWithLabels*>(segment->owner());
        line->setArrowHeight(ahHeight);
        line->setArrowWidth(ahWidth);
      }
    }    
  }
}







