/***************************************************************************
    copyright            : (C) 2004 - 2005 by Marcin Szpyrka
    email                : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CNETVIEW_H
#define CNETVIEW_H

#include <qcanvas.h>
#include <qpainter.h>
#include <qcolor.h>
#include "../model/cnet.h"
#include "cellipsewithlabels.h"
#include "crectanglewithlabels.h"
#include "clinewithlabels.h"
#include <qpopupmenu.h>
#include <qaction.h>
#include "chandle.h"


enum CNetViewMode
{ vmSelect = 0, vmAddPlace = 1, vmAddTrans = 2, vmAddArc = 3,
  vmAutomaticRun = 4, vmInteractiveRun = 5, vmStopped = 6 };


/**
  The CNetView class provides a canvas management for Petri nets editor.
  The class inherits methods from QCanvasiView.
*/

class CNetView : public QCanvasView
{
  Q_OBJECT

private:
  // node sizes
  int  plWidth;
  int  plHeight;
  int  trWidth;
  int  trHeight;
  int  linWidth;
  int  ahWidth;
  int  ahHeight;

  // elements counters
  int  placeID;
  int  transID;

  // last mouse position
  int mouseBufferX;
  int mouseBufferY;

  CNetViewMode viewMode;
  CNet*        net;
  QString      netName;

  QCanvasItem* startElem;

  // pomocnicze zmienne
  CRectangleWithLabels* rectangle;
  CEllipseWithLabels*   ellipse;
  CLineWithLabels*      line;
  CHandle*              curHand;
  int                   arcPoints; // sluzy do zapamietania ile punktow juz wybrano
                                   // przy rysowaniu luku i jaki bedzie jego kierunek
  bool                  moved;
  
  bool withPriorities;
  bool withTime;

  QAction *addBreakPointAction;
  QAction *deleteBreakPointAction;

public:
  CNetView(QWidget * parent, QCanvas * canvas, CNet * model);

  /** Destroys the CNetView. */
  ~CNetView(){}

  void contentsContextMenuEvent(QContextMenuEvent *e);
  void contentsMousePressEvent(QMouseEvent * e);
  void contentsMouseReleaseEvent(QMouseEvent * e);
  void contentsMouseMoveEvent(QMouseEvent * e);

  /** Returns the net view mode. */
  CNetViewMode mode() const { return viewMode; }

  void setMode(CNetViewMode m);

  /** Sets the width of places. */
  void setPlaceWidth(int pw){ plWidth = pw; }

  /** Returns the width of places. */
  int placeWidth(){ return plWidth; }

  /** Sets the height of places. */
  void setPlaceHeight(int ph){ plHeight = ph; }

  /** Returns the height of places. */
  int placeHeight(){ return plHeight; }

  /** Sets the width of transitions. */
  void setTransitionWidth(int tw){ trWidth = tw; }

  /** Returns the width of transitions. */
  int transitionWidth(){ return trWidth; }

  /** Sets the height of transitions. */
  void setTransitionHeight(int th){ trHeight = th; }

  /** Returns the height of transitions. */
  int transitionHeight(){ return trHeight; }

  /** Sets the width of lines. */
  void setLineWidth(int lw){ linWidth = lw; }

  /** Returns the width of lines. */
  int lineWidth(){ return linWidth; }

  /** Sets the width of arrowheads. */
  void setArrowheadWidth(int aw){ ahWidth = aw; }

  /** Returns the width of arrowheads. */
  int arrowheadWidth(){ return ahWidth; }

  /** Sets the height of arrowheads. */
  void setArrowheadHeight(int ah){ ahHeight = ah; }

  /** Returns the height of arrowheads. */
  int arrowheadHeight(){ return ahHeight; }

  /** Sets the net name. */
  void setName(QString nm){ netName = nm; }

  /** Returns the net name. */
  QString name(){ return netName; }
  
  /** Sets initial values of ID numbers for places and transitions. */
  void setID(int places, int transitions){ placeID = places; transID = transitions; }
  
  /** Sets the simulation parameters. */
  void setSimulationParameters(bool priorities, bool time)
  { withPriorities = priorities;
    withTime = time; }

public slots:
  void setSelectMode();
  void setAddPlaceMode();
  void setAddTransMode();
  void setAddArcMode();

  void setAutomaticRunMode();
  void setInteractiveRunMode();
  void stopSimulation();
  void resetInitialMarking();

  void alingmentLeft();
  void alingmentRight();
  void alingmentTop();
  void alingmentBottom();
  void alingmentHorizontalSpread();
  void alingmentVerticalSpread();

  void undo();
  void redo();
  void cut();
  void copy();
  void paste();
  void remove();
  void removeAll();
  void selectAll();
  
  void setCanvasProperties(QString n, int x, int y);
  void setPlaceProperties(QString name, int capacity, int marking, int tag);
  void setTransProperties(QString name, int priority, int time, int tag);
  void setArcProperties(int weight);

  void addBreakPoint();
  void deleteBreakPoint();
  
  void updateElements();

signals:
  void canvasSelected(CNetView *);
  void placeSelected(CEllipseWithLabels*);
  void transSelected(CRectangleWithLabels*);
  void arcSelected(CLineWithLabels*);
	void updateClock(int);
};

#endif

