/***************************************************************************
    copyright            : (C) 2004 - 2005 by Marcin Szpyrka
    email                : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "crectangle.h"


//--------------------------------------------------------------------------
/** Draw the shape. The \a pen and \a brush of \a the painter are already set
  to \a pen() and \a brush() (\a setPen() and \a setBrush() may be used. */

void CRectangle::draw(QPainter & painter)
{
  setBrush(QBrush(Qt::white, Qt::SolidPattern));
  
  if(isEnabled())
  {
    setBrush(QBrush(Qt::green, Qt::SolidPattern)); // default green
  }
  
  if(isActive())
  {
    setBrush(QBrush(Qt::yellow, Qt::SolidPattern)); // default yellow
  }


  QCanvasRectangle::draw(painter);

  // if is selected, some extra boxes are drawn
  if(isSelected())
  {
    painter.setBrush(QBrush(QColor(0,0,200), Qt::SolidPattern)); // blue
    painter.setPen(QPen(Qt::NoPen));
    painter.setClipping(false);

    painter.drawRect((int)x()-3, (int)y()-3, 6, 6); //left top
    painter.drawRect((int)(x()+width()/2)-3, (int)y()-3, 6, 6);
    painter.drawRect((int)(x()+width())-6+3, (int)y()-3, 6, 6);
    painter.drawRect((int)x()-3, (int)(y()+height()/2)-3, 6, 6);
    painter.drawRect((int)(x()+width())-6+3,(int)(y()+height()/2)-3, 6, 6);
    painter.drawRect((int)x()-3, (int)(y()+height())-6+3, 6, 6);
    painter.drawRect((int)(x()+width()/2)-3, (int)(y()+height())-6+3, 6, 6);
    painter.drawRect((int)(x()+width())-6+3, (int)(y()+height())-6+3, 6, 6);

    painter.setPen(QPen());
    painter.setBrush(QBrush());
  }
}


//--------------------------------------------------------------------------
/** Sets the color of the rectangle. */

void CRectangle::setColor(const QColor & c)
{
  QPen p = pen();
  p.setWidth(pen().width());
  p.setColor(c);
  setPen(p);
}


//--------------------------------------------------------------------------
/** Returns the points bounding the rectangle. */

QPointArray CRectangle::areaPoints () const
{
  QPointArray points;
  points =
    QPointArray(QRect(int(x()-4), int(y()-4), int(width()+8), int(height()+8)));
  return points;
}







