/***************************************************************************
    copyright            : (C) 2004 - 2005 by Marcin Szpyrka
    email                : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CRECTANGLE_H
#define CRECTANGLE_H

#include <qcanvas.h>
#include <qpainter.h>
#include <qcolor.h>
#include "celement.h"


/**
  The CRectangle class provides a rectangle on a QCanvas.
  The class inherits methods from QCanvasRectangle.
  The method \a drawShape() has been redefined.
*/

class CRectangle : public QCanvasRectangle, public CElement
{
public:

  enum { RTTI = 10001 };

  /**
    Constructs a rectangle at position (0,0) with both
    width and height set to 32 pixels on \a canvas.
  */
  CRectangle(int rtag, QCanvas* canvas) : QCanvasRectangle(canvas), 
    CElement(rtag){}

  /**
    Constructs a rectangle at position (\a x, \a y) and
    size \a width by \a height, on \a canvas.
  */
  CRectangle(int rtag, int x, int y, int width, int height, QCanvas* canvas) :
      QCanvasRectangle(x, y, width, height, canvas), CElement(rtag){}

  /** Destroys the CRectangle. */
  virtual ~CRectangle(){ hide(); }

  /** Returns CRectangle class RTTI */
  virtual int rtti() const { return RTTI; }

  void draw(QPainter & painter);
  QPointArray areaPoints() const;
  void setColor(const QColor & c);
};

#endif





