/***************************************************************************
    copyright            : (C) 2004 - 2005 by Marcin Szpyrka
    email                : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "crectanglewithlabels.h"


//--------------------------------------------------------------------------
/** Adds the label \a lab to the list of the object labels and returns \a true
  or returns \a false if a label with the same tag already exists. */
  
bool CRectangleWithLabels::addLabel(CLabel* lab)
{
  int ltag = lab->tag();
  CLabel* i;

  // check whether such tag already exists
  for(i = labels.first(); i; i = labels.next())
  {
    if(i->tag() == ltag)
      return false;
  }

  labels.append(lab);
  return true;	
}


//--------------------------------------------------------------------------
/**
  Creates a new label, adds it to the list of the object labels and returns \a true
  or returns \a false if a label with the same tag already exists.
  \param ltag the label tag,
  \param x the horizontal position of the label,
  \param y the horizontal position of the label,
  \param text the label text,
  \param yes if \a yes is \a true the label is placed inside a box
*/

bool CRectangleWithLabels::addLabel(int ltag, int x, int y, QString text, bool yes)
{
  CLabel* i;

  // check whether such tag already exists
  for(i = labels.first(); i; i = labels.next())
  {
    if(i->tag() == ltag)
      return false;
  }

  i = new CLabel(ltag, x, y, text, yes, canvas());
  labels.append(i);
  return true;
}


//--------------------------------------------------------------------------
/** Removes the label with \a ltag tag from the list of the object labels
  and returns \a true or returns \a false if there is no such a label. */
  
bool CRectangleWithLabels::removeLabel(int ltag)
{
  CLabel* i;

  // check whether such tag exists
  for(i = labels.first(); i; i = labels.next())
  {
    if(i->tag() == ltag)
    {
      labels.remove(i);
      return true;
    }
  }
  
  return false;
}


//--------------------------------------------------------------------------
/** Removes the label \a lab from the list of the object labels
  and returns \a true or returns \a false if there is no such a label. */
  
bool CRectangleWithLabels::removeLabel(CLabel* lab)
{
  CLabel* i;

  // check whether such tag already exists
  for(i = labels.first(); i; i = labels.next())
  {
    if(i == lab)
    {
      labels.remove(i);
      return true;
    }
  }
  
  return false;
}


//--------------------------------------------------------------------------
/**	Removes all labels. */

void CRectangleWithLabels::removeAllLabels()
{
  labels.setAutoDelete(true);
  labels.clear();
  labels.setAutoDelete(false);
}

	
//--------------------------------------------------------------------------
/**	Moves the object to the absolute position (\a nx, \a ny). All labels are
	moved so as to preserve distances between the object and its labels. */
  
void CRectangleWithLabels::move(double nx, double ny)
{
	double x1, y1;
	
	// compute the move distance
	x1 = x() - nx;
	y1 = y() - ny;
	
	CRectangle::move(nx, ny);
	
	CLabel* i;
	
	// move all labels
	for(i = labels.first(); i; i = labels.next()) 
	{
		i->moveBy(x1, y1);
	}
	canvas()->setAllChanged();
}
	

//--------------------------------------------------------------------------
/** Moves the object and all its labes relative to its current 
  position by (dx, dy). */
  
void CRectangleWithLabels::moveBy(double dx, double dy)
{
  CRectangle::moveBy(dx, dy);

  CLabel* i;

  // move all labels
  for(i = labels.first(); i; i = labels.next())
  {
    i->moveBy(dx, dy);
  }
}
	

//--------------------------------------------------------------------------
/**	Makes the rectangle and its labels visible.  */	

void CRectangleWithLabels::show()
{
	CRectangle::show();
	
	CLabel* i;
	
	// show all labels
	for(i = labels.first(); i; i = labels.next()) 
	{
		i->show();
	}
}

















