/***************************************************************************
    copyright            : (C) 2004 - 2005 by Marcin Szpyrka
    email                : mszpyrka@agh.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CRECTANGLEWITHLABELS_H
#define CRECTANGLEWITHLABELS_H

#include "crectangle.h"
#include "clabel.h"
#include <qptrlist.h>
#include "../model/ctransition.h"


/**
  The CRectangleWithLabels class provides a rectangle on a QCanvas
  with a list of labels connected with it.
*/

class CRectangleWithLabels : public CRectangle
{
protected:
  QPtrList<CLabel> labels;
public:
  enum { RTTI = 10004 };

  /**
    Constructs a rectangle at position (0,0) with both
    width and height set to 32 pixels on \a canvas.
  */
  CRectangleWithLabels(int rtag, QCanvas* canvas) :
      CRectangle(rtag,canvas){}

  /**
    Constructs a rectangle at position (\a x, \a y) and
    size \a width by \a height, on \a canvas.
  */
  CRectangleWithLabels(int rtag, int x, int y, int width,
                       int height, QCanvas* canvas) :
  CRectangle(rtag, x, y, width, height, canvas){ }

	/** Destroys the rectangle. */
	~CRectangleWithLabels(){ }

  /** Returns CRectangleWithLabels class RTTI */
  int rtti() const { return RTTI; }

  // adding labels
  bool addLabel(CLabel* lab);
  bool addLabel(int ltag, int x, int y, QString text, bool yes);

  // removing labels
  bool removeLabel(int ltag);
  bool removeLabel(CLabel* lab);
  void removeAllLabels();

  // access to labels
  /**
  Returns a pointer to the label at position \a index in the list,
  or 0 if the index is out of range. The valid range is 0..(labelsCount() - 1)
  inclusive.
  */
  CLabel* label(int index) { return labels.at(index); }

  /** Returns the number of labels. */
  int labelsCount() const { return labels.count(); };

  // moving elements
  void move(double nx, double ny);
  void moveBy(double dx, double dy);
  void show();
};

#endif












